function gtRenumberInterlaced()

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load('parameters.mat');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters=[];
load('parameters.mat');

% initial checks
if parameters.acq.interlaced_turns==0
    disp('Not an interlaced scan!')
    return
end

interlacedscan=parameters.acq.interlaced_turns+1;
% if all interlaced scans are 360 degrees...
totproj=2*parameters.acq.nproj;
imageperturn=totproj/(interlacedscan);

if isfield(parameters.prep,'renumbered') && parameters.prep.renumbered==true
    disp('The interlaced renumbering has already been done! ');
    return
else
    if length(dir([parameters.acq.dir '/1_preprocessing/full/full*edf']))<totproj
        disp('not all of the full images exist!')
    else
        % do renumbering
        parameters.prep.renumbered=true;
        disp('The interlaced renumbering will be processed now! ');
        save([parameters.acq.dir '/parameters.mat'],'parameters');
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% renumber full images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd('1_preprocessing/full');

disp(' moving original full ...');
% intermediate - move to a tmp directory
mkdir('tmp');
% have to do this 1000 images at a time - faster than individually
for i=0:floor(totproj/1000)
    orig=sprintf('full%d*.edf',i);
    [s,msg]=movefile(orig,'tmp'); disp(msg)
    clear s msg
end

% final renumbering
disp(' renaming full ...');
for i=0:2*parameters.acq.nproj-1
    orig=sprintf('tmp/full%04d.edf',i);
    j=interlacedscan*mod(i,imageperturn)+floor(i/imageperturn);
    dest=sprintf('full%04d.edf',j);
    [s,msg]=movefile(orig,dest); disp(msg)
    clear s msg
end

% clean-up tmp directory
check=dir('tmp/*.edf');
if isempty(check)
    delete('tmp');
end
cd(parameters.acq.dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% renumber ext images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist('1_preprocessing/ext/ext0000.edf','file')

    cd('1_preprocessing/ext');
    disp('copying original ext ...');
    % intermediate - move to a tmp directory
    mkdir('tmp');
    % have to do this 1000 images at a time - faster than individually
    for i=0:floor(totproj/1000)
        orig=sprintf('ext%d*.edf',i);
        [s,msg]=movefile(orig,'tmp'); disp(msg)
        clear s msg
    end

    % final renumbering
    disp(' renaming ext ...');
    for i=0:2*parameters.acq.nproj-1
        orig=sprintf('tmp/ext%04d.edf',i);
        j=interlacedscan*mod(i,imageperturn)+floor(i/imageperturn);
        dest=sprintf('ext%04d.edf',j);
        [s,msg]=movefile(orig,dest); disp(msg)
        clear s msg
    end

    % clean-up tmp directory
    check=dir('tmp/*.edf');
    if isempty(check)
        delete('tmp');
    end
    cd(parameters.acq.dir);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% renumber abs images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist('1_preprocessing/abs/abs0000.edf','file')

    cd('1_preprocessing/abs');
    % intermediate - move to a tmp directory
    mkdir('tmp');
    % have to do this 1000 images at a time - faster than individually
    for i=0:floor(totproj/1000)
        orig=sprintf('abs%d*.edf',i);
        [s,msg]=movefile(orig,'tmp'); disp(msg)
        clear s msg    
    end

    % final renumbering
    disp('renaming abs ...');
    for i=0:2*parameters.acq.nproj-1
        orig=sprintf('tmp/abs%04d.edf',i);
        j=interlacedscan*mod(i,imageperturn)+floor(i/imageperturn);
        dest=sprintf('abs%04d.edf',j);
        [s,msg]=movefile(orig,dest); disp(msg)
        clear s msg    
    end

    % clean-up tmp directory
    check=dir('tmp/*.edf');
    if isempty(check)
        delete('tmp');
    end
    cd(parameters.acq.dir);
end

end %end of function