function acq = gtLoadAcquisitionXML(xmlfname, interactive)
% FUNCTION gtLoadAcquisitionXML  Try to read the .xml file that may be present
%     params_acq = gtLoadAcquisitionXML(xmlfname, interactive)
%     --------------------------------------------------------
%
%     Version 002 15/05/2012 by LNervo
%       Split acq parameters from parameters read from xml file

    % try to read the .xml file that may be present
    acq = [];
    
    if exist(xmlfname,'file')
        disp('Reading parameters from xml file');
        tmpxml = xml_read(xmlfname);
        tmpxml = tmpxml.acquisition;

        acq.name          = tmpxml.scanName; % shouldn't change!
        acq.date          = tmpxml.date;
        acq.xdet          = tmpxml.projectionSize.DIM_1; % pixels
        acq.ydet          = tmpxml.projectionSize.DIM_2; % pixels
        acq.nproj         = tmpxml.tomo_N/2;
        acq.refon         = tmpxml.ref_On;
        acq.nref          = tmpxml.ref_N;
        acq.ndark         = tmpxml.dark_N;
        acq.pixelsize     = tmpxml.pixelSize/1000; % mm
        acq.count_time    = tmpxml.ccdtime;
        acq.energy        = tmpxml.energy; % keV
        acq.dist          = tmpxml.distance; % mm
        acq.sensortype    = tmpxml.cameraName;
        if (isfield(tmpxml, 'scanRange'))
            acq.type      = [num2str(tmpxml.scanRange) 'degree'];
        else
            warning('Could not find scanRange info in xml file, assuming it is 360degree...');
            acq.type      = '360degree';
        end
        acq.detroi_v_off  = tmpxml.projectionSize.ROW_BEG;
        acq.detroi_u_off  = tmpxml.projectionSize.COL_BEG;
    else %if .xml not found
        if interactive
            disp('No .xml file found!');
            disp('-> Please check that you are in the acquistion directory.');
            check = inputwdefault('Do you want to proceed without the xml file? [y/n]', 'n');
            if strcmpi(check, 'n')
                gtError('XML:no_xml_file', ...
                    'There is no .xml file and the user chose to quit');
            end
        end
        % guess some default values
        acq.date          = date();
        acq.xdet          = 2048;
        acq.ydet          = 2048;
        acq.nproj         = 7200;
        acq.refon         = 200;
        acq.nref          = 5;
        acq.ndark         = 11;
        acq.pixelsize     = 0.0014;
        acq.count_time    = 2;
        acq.energy        = 20;
        acq.dist          = 5;
        acq.sensortype    = 'frelon';
        acq.type          = '360degree';
        acq.detroi_u_off  = 0;
        acq.detroi_v_off  = 0;
    end
    
end % end of function
