function im = gtPlaceDifspotinFull(struct_id, parameters, varargin)
% GTPLACEDIFSPOTINFULL  Draws all the difspots together
%     im = gtPlaceDifspotinFull(struct_id, parameters, varargin)
%     ----------------------------------------------------------
%     Returns the full image requested, cropped to the specified difspot
%     bounding box
%
%     INPUT:
%       struct_id  = <double>   list of difspot (i.e. grain{grainid}.difspotID)
%       parameters = <struct>   parameters.mat
%
%     OPTIONAL INPUT (parse by pairs):
%       connected  = <logical>  already connected to the database {false}
%       save       = <logical>  desire to save the produced image {false}
%       replace    = <logical>  replace existing <name>_summed.edf file {false}
%       draw       = <logical>  show image {false}
%       id         = <double>   grain id {0}
% 
%     OUTPUT:
%       im         = <double>   image with all the difspots summed
%
%     Version 002 13-12-2013 by LNervo


app.id        = 0;
app.draw      = false;
app.connected = false;
app.save      = false;
app.replace   = false;
app = parse_pv_pairs(app,varargin);


if ~exist('parameters','var') || isempty(parameters)
    parameters = [];
    load('parameters.mat');
end 

name = parameters.acq.name;
namefull = [name '_grain' num2str(app.id) '_summed.edf'];

if exist(namefull,'file')
    if (app.replace)
        im = edf_read(namefull);
        summed = true;
    else
        summed = false;
    end
else
    summed = false;
end

if (~summed)
    im = zeros(parameters.acq.ydet,parameters.acq.xdet);
    
    % connect to database
    if (~app.connected)
        gtDBConnect();
        app.connected = true;
    end

    for ii=1:length(struct_id)

        full = zeros(size(im));
        [dif,bb] = gtGetSummedDifSpot(struct_id(ii),parameters,app.connected);
        
        full = gtPlaceSubImage2(dif,full,bb(1),bb(2));
        im = im+full;
    end
    
end % end if ~summed

if (app.draw)
    figure();
    imshow(im,[]);
end

if (app.save) && (~summed)
    edf_write(im,namefull);
end

end  % end of function
