function [gr, refs] = gtComputeIncomingBeamIntensity(gr, parameters, refs)
% FUNCTION [gr, refs] = gtComputeIncomingBeamIntensity(gr, parameters, refs)

    if (isfield(gr.proj, 'ondet'))
        gr_included = gr.proj.ondet(gr.proj.included);
    else
        gr_included = gr.ondet(gr.included);
    end

    omega_step = gtGetOmegaStepDeg(parameters);

    omegas = gr.allblobs.omega(gr_included);
    gc_det_pos = gtGeoGrainCenterSam2Det(gr.center, omegas, parameters);
    rounded_gc_det = round(gc_det_pos);

    img_nums = omegas / omega_step;

    if (parameters.acq.interlaced_turns > 0)
        error('gtComputeIncomingBeamIntensity:wrong_conf', ...
            'Interlaced turns not supported, yet!')

        % TO BE FINISHED
        min_ws = floor(img_nums);
        max_ws = min_ws + 1;

        min_cs = max_ws - img_nums;
        max_cs = 1 - min_cs;

        min_min_ref_ws = floor(min_ws / parameters.acq.refon) + 1;
        min_max_ref_ws = floor(max_ws / parameters.acq.refon) + 1;
        max_min_ref_ws = min_min_ref_ws + 1;
        max_max_ref_ws = min_max_ref_ws + 1;
    else
        min_ref_ws = floor(img_nums / parameters.acq.refon) + 1;
        max_ref_ws = min_ref_ws + 1;

        min_cs = (max_ref_ws - (img_nums / parameters.acq.refon) - 1);
        max_cs = 1 - min_cs;
    end

    if (~exist('refs', 'var'))
        refs = load_reference_imgs(parameters);
    end

    max_selection = max_cs > 1e-4;
    min_indx = sub2ind(size(refs), rounded_gc_det(:, 2), rounded_gc_det(:, 1), min_ref_ws);
    max_indx = sub2ind(size(refs), rounded_gc_det(max_selection, 2), rounded_gc_det(max_selection, 1), max_ref_ws(max_selection));
    min_vals = refs(min_indx) .* min_cs;
    max_vals = refs(max_indx) .* max_cs(max_selection);

    int_profile = min_vals;
    int_profile(max_selection) = int_profile(max_selection) + max_vals;

    gr.beam_intensity = int_profile;

%     ref0 = refs;
%     ref0(min_indx) = -20000;
%     ref0(max_indx) = -20000;
%     GtVolView(ref0)
end

function refs = load_reference_imgs(parameters)
    img_nums = 0:parameters.acq.refon:(parameters.acq.nproj*2);
%     refs = zeros(parameters.acq.ydet, parameters.acq.ydet, numel(img_nums));
    base_dir = fullfile('0_rawdata', parameters.acq.name);
    img_0 = fullfile(base_dir, sprintf('refHST%04d.edf', 0));
    info = edf_info(img_0);
    fprintf(' - loading ref images: ')
    c = tic();
    tot_images = numel(img_nums);
    for ii = tot_images:-1:1
        num_chars = fprintf('%02d/%02d', tot_images - ii + 1, tot_images);
        img_name = fullfile(base_dir, sprintf('refHST%04d.edf', img_nums(ii)));
        refs(:, :, ii) = edf_read(img_name, [], false, info);
        fprintf(repmat('\b', [1, num_chars]))
    end
    fprintf('(%02d) in %f seconds.\n', tot_images, toc(c));
end