function abs = gtAstraAbsorptionVolume(parameters)
% GTASTRAABSORPTIONVOLUME reconstruct absorption volume using 3D SIRT or 2D FBP
%     abs = gtAstraAbsorptionVolume(parameters)
%     -----------------------------------------
%     Checks if absorption_stack.mat exists (will create it otherwise)
%     Checks if absorption_mask.mat exists (will create it otherwise)
%     Outputs reconstructed volume into file absorption.mat
%
%     parameters.rec.method : '3DART' / '2DFBP'
%
%     Version 002 26-09-2012 by LNervo
%       Formatting, cleaning, commenting


    if ~exist('parameters','var') || isempty(parameters)
        load('parameters.mat');
    end

    projname = fullfile(parameters.acq.dir, 'absorption_stack.mat');

    if (exist(projname, 'file'))
        check = inputwdefault('Absorption stack already exists - do you want to (re)-create it? [y/n]', 'n');

        if strcmpi(check, 'n')
            fprintf('Loading absorption stack..')
            c = tic();
            proj = load(projname);
            fprintf('\b\b: Done. (%f seconds)\n', toc(c))
        else
            proj = gtAstraPrepareAbsorptionStack(parameters);
        end
    else
        proj = gtAstraPrepareAbsorptionStack(parameters);
    end

    if (isfield(parameters.rec, 'absorption'))
        rec_abs = parameters.rec.absorption;
    else
        % Old style parameters
        warning('Old "rec" parameters')
        rec_abs = parameters.rec;
        rec_abs.algorithm = rec_abs.method;
    end
    proj.num_iter = rec_abs.num_iter;

    fprintf('Reconstructing..')
    c = tic();
    is_cone = isfield(parameters.labgeo, 'sourcepoint') && (~isempty(parameters.labgeo.sourcepoint));
    switch (rec_abs.algorithm)
        case {'3DART', 'SIRT'}
            abs = gtAstra3D(proj, 'is_cone', is_cone);
        case {'3DTV'}
            abs = gtAstra3DTV(proj, 'is_cone', is_cone);
        case '2DFBP'
            abs = gtAstra_FBP2D(parameters, proj);
        otherwise
            error('gtAstraAbsorptionVolume:wrong_algorithm', ...
                'Algorithm for reconstruction not known. Please check parameters file');
    end
    fprintf('\b\b: Done. (%f seconds)\n', toc(c))
end




