function rec_opts = gtReconstruct6DGetParamenters(parameters)
% FUNCTION rec_opts = gtReconstruct6DGetParamenters(parameters)
%

    rec = parameters.rec;
    if (~isfield(rec, 'grains'))
        error([mfilename ':bad_structure'], ...
            'The parameters are too old or malformed: there is no ".grains" in .rec!')
    end

    num_iter = rec.grains.num_iter;

    if (~isfield(rec.grains, 'options') || isempty(rec.grains.options))
        warning('gtReconstruct6DGetParamenters:wrong_parameters', ...
            'The rec.grains structure doesn''t seem to be valid. Falling back to defaults')
        rec.grains = gtRecGrainsDefaultParameters(rec.grains.algorithm);
    end

    rec_opts = rec.grains.options;
    rec_opts.num_iter = num_iter;
    rec_opts.algorithm = rec.grains.algorithm;

    if (~isfield(rec_opts, 'volume_downscaling') ...
            || isempty(rec_opts.volume_downscaling))
        rec_opts.volume_downscaling = 1;
    end
    if (~isfield(rec_opts, 'ospace_super_sampling') ...
            || isempty(rec_opts.ospace_super_sampling))
        if (~isfield(rec_opts, 'super_sampling') ...
                || isempty(rec_opts.super_sampling))
            rec_opts.ospace_super_sampling = 1;
        else
            rec_opts.ospace_super_sampling = rec_opts.super_sampling;
        end
    end
    if (~isfield(rec_opts, 'rspace_super_sampling') ...
            || isempty(rec_opts.rspace_super_sampling))
        rec_opts.rspace_super_sampling = 1;
    end
    if (~isfield(rec_opts, 'ospace_oversize') ...
            || isempty(rec_opts.ospace_oversize))
        rec_opts.ospace_oversize = 1.1;
    end
    if (~isfield(rec_opts, 'rspace_oversize') ...
            || isempty(rec_opts.rspace_oversize))
        rec_opts.rspace_oversize = 1.2;
    end
    if (~isfield(rec_opts, 'use_predicted_scatter_ints') ...
            || isempty(rec_opts.use_predicted_scatter_ints))
        rec_opts.use_predicted_scatter_ints = false;
    end
    if (~isfield(rec_opts, 'shape_functions_type') ...
            || isempty(rec_opts.shape_functions_type))
        rec_opts.shape_functions_type = 'none';
    end
    if (~isfield(rec_opts, 'detector_norm') ...
            || isempty(rec_opts.detector_norm))
        rec_opts.detector_norm = 'KL';
    end
    if (~isfield(rec_opts, 'tv_norm') ...
            || isempty(rec_opts.tv_norm))
        rec_opts.detector_norm = 'l12';
    end
    if (~isfield(rec_opts, 'tv_strategy') ...
            || isempty(rec_opts.tv_strategy))
        rec_opts.tv_strategy, 'groups';
    end
end
