function algo = gtReconstruct6DLaunchAlgorithm(sampler, rec_opts, parameters, varargin)
% FUNCTION algo = gtReconstruct6DLaunchAlgorithm(sampler, rec_opts, parameters, varargin)
%

    conf = struct('det_index', 1, 'ODF', []);
    conf = parse_pv_pairs(conf, varargin);

    rec_factory = Gt6DReconstructionAlgorithmFactory(parameters, ...
        'volume_downscaling', rec_opts.volume_downscaling, ...
        'rspace_super_sampling', rec_opts.rspace_super_sampling, ...
        'rspace_oversize', rec_opts.rspace_oversize, ...
        'use_predicted_scatter_ints', rec_opts.use_predicted_scatter_ints, ...
        'shape_functions_type', rec_opts.shape_functions_type, ...
        'det_index', conf.det_index );

    algo = rec_factory.getReconstructionAlgo(sampler, rec_opts.num_interp);

    % Adding extra constraints
    algo.ODF = conf.ODF;

    % Adding extra parameters
    algo.detector_norm = rec_opts.detector_norm;
    algo.tv_norm = rec_opts.tv_norm;
    algo.tv_strategy = rec_opts.tv_strategy;

    geom_memory = sum(arrayfun(@(x)x.get_memory_consumption_geometry(), sampler));
    grain_memory = sum(arrayfun(@(x)x.get_memory_consumption_graindata(), sampler));

    mem_cons = [ ...
        (algo.get_peak_memory_consumption(rec_opts.algorithm) + geom_memory + grain_memory), ...
        algo.get_memory_consumption_volumes(), ...
        algo.get_memory_consumption_blobs(), ...
        algo.get_memory_consumption_sinograms(), ...
        geom_memory, ...
        grain_memory ];
    fprintf(['Estimated memory consumption: %d MB (Volumes %d MB, Blobs %d MB, ' ...
        'Sinograms %d MB, Geometry %d MB, Grain data %d MB)\n'], ...
        round(mem_cons / (2^20)))

    algo.cp(upper(rec_opts.algorithm), rec_opts.num_iter, rec_opts.lambda);
end
