/*
 * gt6DBlobsToSingleSino_c.cpp
 *
 *  Created on: Oct 30, 2014
 *      Author: vigano
 */

#include "gt6DBlobsSinogramsTransforms.h"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs < 3) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "Not enough arguments!");
    return;
  }

  const mxArray * const blobs = prhs[1];
  const mxArray * const coeffs = prhs[2];

  if (!GT6D::check_coeffs(coeffs))
  {
    return;
  }

  if (!mxIsNumeric(prhs[0])) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The first argument should be an array (Sinogram sizes)");
    return;
  }
  if (!mxIsCell(blobs)) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The second argument should be a Cell array (Blobs)");
    return;
  }

  const mxClassID class_id = getMxClassOfCell(blobs);
  if (class_id == mxUNKNOWN_CLASS)
  {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The blobs need to be a non empty Cell array of coherent floating point types");
    return;
  }

  if (nrhs >= 4)
  {
    initialize_multithreading(mxGetScalar(prhs[3]));
  }
  else
  {
    initialize_multithreading();
  }

  mxArray * const sino = GT6D::allocate_sino_from_dims(prhs[0], class_id);
  if (!sino)
  {
    return;
  }
  plhs[0] = sino;

  if (!GT6D::check_blobs_and_single_sinogram(sino, blobs, coeffs))
  {
    return;
  }

  switch (class_id)
  {
    case mxDOUBLE_CLASS:
    {
      {
        GT6D::do_blobs_to_single_sinogram<double>(sino, blobs, coeffs);
      }
      break;
    }
    case mxSINGLE_CLASS:
    {
      {
        GT6D::do_blobs_to_single_sinogram<float>(sino, blobs, coeffs);
      }
      break;
    }
    default:
    {
      mexErrMsgIdAndTxt(GT6D::transforms_error_id,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


