/*
 * gt6DSingleSinoToBlobs_c.cpp
 *
 *  Created on: Oct 31, 2014
 *      Author: vigano
 */

#include "gt6DBlobsSinogramsTransforms.h"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs < 3) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "Not enough arguments!");
    return;
  }

  const mxArray * const in_blobs = prhs[0];
  const mxArray * const sino = prhs[1];
  const mxArray * const coeffs = prhs[2];

  if (!GT6D::check_coeffs(coeffs))
  {
    return;
  }

  if (!mxIsCell(in_blobs)) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The first argument should be a Cell array (Blobs)");
    return;
  }
  if (!mxIsNumeric(sino)) {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The second argument should be an array (Sinogram)");
    return;
  }

  if (nrhs >= 4)
  {
    initialize_multithreading(mxGetScalar(prhs[3]));
  }
  else
  {
    initialize_multithreading();
  }

  if (!GT6D::check_blobs_and_single_sinogram(sino, in_blobs, coeffs))
  {
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(in_blobs);

  const mxClassID class_id = getMxClassOfCell(in_blobs);
  if (class_id != mxGetClassID(sino))
  {
    mexErrMsgIdAndTxt(GT6D::transforms_error_id,
        "The sinogram needs to be an array of coherent floating point type, with the arrays in the Blob cell");
    return;
  }

  switch (class_id)
  {
    case mxDOUBLE_CLASS:
    {
      {
        GT6D::do_single_sinogram_to_blobs<double>(plhs[0], sino, coeffs);
      }
      break;
    }
    case mxSINGLE_CLASS:
    {
      {
        GT6D::do_single_sinogram_to_blobs<float>(plhs[0], sino, coeffs);
      }
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(GT6D::transforms_error_id,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(GT6D::transforms_error_id,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


