/*
 * gt6DUpdateDualDetector_c.cpp
 *
 *  Created on: Oct 29, 2014
 *
 * Nicola Vigano', 2014, INSA Lyon / ESRF ID11, vigano@esrf.eu
 */

#include <gt6DUpdateDualDetectorOps.h>

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs < 5) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "Not enough arguments!");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "The first argument should be a Cell array (dual variable)");
    return;
  }
  if (!mxIsCell(prhs[1])) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "The second argument should be a Cell array (computed blobs)");
    return;
  }
  if (!mxIsCell(prhs[2])) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "The third argument should be a Cell array (real blobs)");
    return;
  }
  if (!mxIsCell(prhs[3])) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "The fourth argument should be a Cell array (sigma1)");
    return;
  }
  if (!mxIsCell(prhs[4])) {
    mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
        "The fifth argument should be a Cell array (sigma1_1)");
    return;
  }

  if (nrhs >= 6)
  {
    initialize_multithreading(mxGetScalar(prhs[5]));
  }
  else
  {
    initialize_multithreading();
  }

  if (!GT6D::check_dual_detector_arguments(prhs[0], prhs[1], prhs[2], prhs[3], prhs[4]))
  {
    return;
  }

//  cell_allocate(plhs[0], prhs[0]);
  plhs[0] = mxCreateSharedDataCopy(prhs[0]);

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      GT6D::update_dual_detector<double> func;
      GT6D::dual_detector_iteration<double>(plhs[0], prhs[0], prhs[1], prhs[2], prhs[3], prhs[4], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      GT6D::update_dual_detector<float> func;
      GT6D::dual_detector_iteration<float>(plhs[0], prhs[0], prhs[1], prhs[2], prhs[3], prhs[4], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(GT6D::dual_detector_error_id,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


