/*
 * gt6DUpdateDualL1_c.cpp
 *
 *  Created on: Oct 31, 2014
 *      Author: vigano
 */

#include <gt6DUpdateDualL1Ops.h>

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs < 3) {
    mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
        "Not enough arguments!");
    return;
  }

  const mxArray * const dual = prhs[0];
  const mxArray * const new_enh_sol = prhs[1];
  const mxArray * const sigma_a = prhs[2];

  if (!mxIsCell(dual)) {
    mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
        "The first argument should be a Cell array (dual variable)");
    return;
  }
  if (!mxIsCell(new_enh_sol)) {
    mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
        "The second argument should be a Cell array (New Enhancement of Solution)");
    return;
  }
  if (!mxIsNumeric(sigma_a)) {
    mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
        "The third argument should be a scalar (sigma)");
    return;
  }
  if (!mxIsDouble(sigma_a) && !mxIsSingle(sigma_a)) {
    mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
        "The third argument should be either a 'double' or a 'single'");
    return;
  }

  double sigma;
  if (mxIsDouble(sigma_a)) {
    sigma = *mxGetPr(sigma_a);
  } else {
    sigma = *(float *)mxGetData(sigma_a);
  }

  if (nrhs >= 5)
  {
    initialize_multithreading(mxGetScalar(prhs[4]));
  }
  else
  {
    initialize_multithreading();
  }

  const mxClassID data_type = GT6D::check_dual_l1_arguments(dual, new_enh_sol);
  if (data_type == mxUNKNOWN_CLASS)
  {
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(dual);

  switch (data_type)
  {
    case mxDOUBLE_CLASS:
    {
      GT6D::update_dual_l1<double> func(sigma);
      cell_iteration<double, false>(plhs[0], prhs[1], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      GT6D::update_dual_l1<float> func(sigma);
      cell_iteration<float, false>(plhs[0], prhs[1], func);
      break;
    }
    default:
    {
      mexErrMsgIdAndTxt(GT6D::dual_l1_error_id,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}
