/*
 * internal_cell_copy.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_copy:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 1)
  {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array");
    return;
  }
  if (!mxIsCell(prhs[0]))
  {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      inner_assign<double> func;
      cell_iteration<double, true>(plhs[0], prhs[0], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_assign<float> func;
      cell_iteration<float, true>(plhs[0], prhs[0], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}



