/*
 * internal_cell_div.cpp
 *
 *  Created on: Jul 12, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_div_copy:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 2) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide two cell arrays");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a Cell array");
    return;
  }
  const mwSize numElems1 = mxGetNumberOfElements(prhs[0]);
  const mwSize numElems2 = mxGetNumberOfElements(prhs[1]);

  if (numElems1 != numElems2) {
    mexErrMsgIdAndTxt(errorID,
        "The Cell arrays should have the same size");
    return;
  }

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  if (firstClass != getMxClassOfCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The arguments need to be Cell arrays of coherent floating point types");
    return;
  }

  switch (firstClass) {
    case mxDOUBLE_CLASS:
    {
      inner_div<double> func;
      cell_iteration_copy<double>(plhs[0], prhs[0], prhs[1], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_div<float> func;
      cell_iteration_copy<float>(plhs[0], prhs[0], prhs[1], func);
      break;
    }
    case mxUNKNOWN_CLASS: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}



