/*
 * internal_cell_exponent.cpp
 *
 *  Created on: Jul 10, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_exponent_assign:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 2) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array and a double");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsDouble(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a double containing the exponent");
    return;
  }

  double exponent;
  if (mxIsDouble(prhs[1])) {
    exponent = *mxGetPr(prhs[1]);
  } else if (mxIsSingle(prhs[1])) {
    exponent = *(float *)mxGetData(prhs[1]);
  } else {
    mexErrMsgIdAndTxt(errorID,
        "The exponent needs to be of a floating point type");
    return;
  }

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      inner_pow<double> func(exponent);
      cell_iteration<double, true>(plhs[0], prhs[0], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_pow<float> func(exponent);
      cell_iteration<float, true>(plhs[0], prhs[0], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}
