/*
 * internal_cell_exponent.cpp
 *
 *  Created on: Jul 10, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_sqrt_assign:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 1) {
    mexErrMsgIdAndTxt(errorID,
        "Wrong number of arguments! you need to provide a cell array");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(prhs[0]);

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      inner_sqrt<double> func;
      cell_iteration<double, false>(plhs[0], prhs[0], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_sqrt<float> func;
      cell_iteration<float, false>(plhs[0], prhs[0], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}
