/*
 * internal_cell_sum_scaled.cpp
 *
 *  Created on: Jul 11, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_sum_scaled_assign:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 3) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide two cell arrays and the scale as a double");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a Cell array");
    return;
  }
  if (!mxIsDouble(prhs[2]) && !mxIsSingle(prhs[2])) {
    mexErrMsgIdAndTxt(errorID,
        "The third argument should be a double or a single");
    return;
  }
  const mwSize numElems1 = mxGetNumberOfElements(prhs[0]);
  const mwSize numElems2 = mxGetNumberOfElements(prhs[1]);

  if (numElems1 != numElems2) {
    mexErrMsgIdAndTxt(errorID,
        "The Cell arrays should have the same size");
    return;
  }

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  if (firstClass != getMxClassOfCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The arguments need to be Cell arrays of coherent floating point types");
    return;
  }

  double scale;
  if (mxIsDouble(prhs[2])) {
    scale = *mxGetPr(prhs[2]);
  } else if (mxIsSingle(prhs[2])) {
    scale = *(float *)mxGetData(prhs[2]);
  } else {
    mexErrMsgIdAndTxt(errorID,
        "The scale needs to be of a floating point type");
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(prhs[0]);

  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      inner_sub_scale<double> func(scale);
      cell_iteration<double, false>(plhs[0], prhs[1], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_sub_scale<float> func(scale);
      cell_iteration<float, false>(plhs[0], prhs[1], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


