/*
 * internal_cell_prod_scalar_assign.cpp
 *
 *  Created on: Jul 12, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_sum_scalar:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 2) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array and a double");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsDouble(prhs[1]) && !mxIsSingle(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a double containing the scalar");
    return;
  }

  double scalar;
  if (mxIsDouble(prhs[1])) {
    scalar = *mxGetPr(prhs[1]);
  } else if (mxIsSingle(prhs[1])) {
    scalar = *(float *)mxGetData(prhs[1]);
  } else {
    mexErrMsgIdAndTxt(errorID,
        "The scale needs to be of a floating point type");
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(prhs[0]);

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      inner_sum_scalar<double> func(scalar);
      cell_iteration<double, false>(plhs[0], prhs[0], func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      inner_sum_scalar<float> func(scalar);
      cell_iteration<float, false>(plhs[0], prhs[0], func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


