/*
 * gt6DOps.h
 *
 *  Created on: Oct 29, 2014
 *      Author: vigano
 */

#ifndef ZUTIL_CXX_INCLUDE_GT6DUPDATEDUALDETECTOROPS_H_
#define ZUTIL_CXX_INCLUDE_GT6DUPDATEDUALDETECTOROPS_H_

#include "internal_cell_defs.h"

#include <algorithm>

namespace GT6D {

#define APPLY_FUNC_5FOLD_DUAL_DETECTOR(shift_val) \
{ \
  const vVvf inV11 = access.load(in1 + shift_val * simd_8.shift);\
  const vVvf inV21 = access.load(in2 + shift_val * simd_8.shift);\
  const vVvf inV31 = access.load(in3 + shift_val * simd_8.shift);\
  const vVvf inV41 = access.load(in4 + shift_val * simd_8.shift);\
  const vVvf inV51 = access.load(in5 + shift_val * simd_8.shift);\
  access.store(out + shift_val * simd_8.shift, func(inV11, inV21, inV31, inV41, inV51));\
}

  const char * dual_detector_error_id = "C_FUN:gt6DUpdateDualDetector:wrong_argument";

  template<typename Type>
  class update_dual_detector {
  public:
    typedef typename SIMDUnrolling<Type>::vVvf vVvf;

    update_dual_detector() { }

    const Type
    operator()(const Type & dual, const Type & blobs,
        const Type & computed_blobs, const Type & sigma1,
        const Type & sigma1_1) const throw()
    {
      return (dual + (computed_blobs - blobs) * sigma1) * sigma1_1;
    }
    const vVvf
    operator()(const vVvf & dual, const vVvf & blobs,
        const vVvf & computed_blobs, const vVvf & sigma1,
        const vVvf & sigma1_1) const throw()
    {
      return (dual + (computed_blobs - blobs) * sigma1) * sigma1_1;
    }
  protected:
  };

  inline bool
  check_dual_detector_arguments(const mxArray * const cells_dual,
      const mxArray * const cells_blobs,
      const mxArray * const cells_computed_blobs,
      const mxArray * const cells_sigma1,
      const mxArray * const cells_sigma1_1)
  {
    const mwSize num_cells = mxGetNumberOfElements(cells_dual);
    if (num_cells != mxGetNumberOfElements(cells_blobs))
    {
      mexErrMsgIdAndTxt(dual_detector_error_id,
              "Blobs should have the same number of cells as the Dual");
      return false;
    }
    if (num_cells != mxGetNumberOfElements(cells_computed_blobs))
    {
      mexErrMsgIdAndTxt(dual_detector_error_id,
              "Computed blobs should have the same number of cells as the Dual");
      return false;
    }
    if (num_cells != mxGetNumberOfElements(cells_sigma1))
    {
      mexErrMsgIdAndTxt(dual_detector_error_id,
              "Sigma1 should have the same number of cells as the Dual");
      return false;
    }
    if (num_cells != mxGetNumberOfElements(cells_sigma1_1))
    {
      mexErrMsgIdAndTxt(dual_detector_error_id,
              "Sigma1_1 should have the same number of cells as the Dual");
      return false;
    }

    for(mwIndex cell_idx = 0; cell_idx < num_cells; cell_idx++)
    {
      const mxArray * const cell_dual = mxGetCell(cells_dual, cell_idx);
      const mxArray * const cell_blob = mxGetCell(cells_blobs, cell_idx);
      const mxArray * const cell_comp_blob = mxGetCell(cells_computed_blobs, cell_idx);
      const mxArray * const cell_sigma1 = mxGetCell(cells_sigma1, cell_idx);
      const mxArray * const cell_sigma1_1 = mxGetCell(cells_sigma1_1, cell_idx);

      const mxClassID type = mxGetClassID(cell_dual);
      const mwSize num_elems = mxGetNumberOfElements(cell_dual);

      if (type != mxGetClassID(cell_blob))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Blobs volumes should have the same type as the Dual");
        return false;
      }
      if (type != mxGetClassID(cell_comp_blob))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Computed volumes blobs should have the same type as the Dual");
        return false;
      }
      if (type != mxGetClassID(cell_sigma1))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Sigma1 volumes should have the same type as the Dual");
        return false;
      }
      if (type != mxGetClassID(cell_sigma1_1))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Sigma1_1 volumes should have the same type as the Dual");
        return false;
      }

      if (num_elems != mxGetNumberOfElements(cell_blob))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Blobs volumes should have the same number of elements as the Dual");
        return false;
      }
      if (num_elems != mxGetNumberOfElements(cell_comp_blob))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Blobs volumes should have the same number of elements as the Dual");
        return false;
      }
      if (num_elems != mxGetNumberOfElements(cell_sigma1))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Sgima1 volumes should have the same number of elements as the Dual");
        return false;
      }
      if (num_elems != mxGetNumberOfElements(cell_sigma1_1))
      {
        mexErrMsgIdAndTxt(dual_detector_error_id,
                "Sigma1_1 volumes should have the same number of elements as the Dual");
        return false;
      }
    }

    return true;
  }

  template<typename Type, typename Function, class AccessType>
  inline void
  detector_dual_inner_cycle_sse(Type * const __restrict out_data,
      const Type * const __restrict in_data1,
      const Type * const __restrict in_data2,
      const Type * const __restrict in_data3,
      const Type * const __restrict in_data4,
      const Type * const __restrict in_data5,
      const mwSize & num_elems, Function & func)
  {
    typedef typename SIMDUnrolling<Type>::vVvf vVvf;

    const SIMDUnrolling<Type> simd_8(8);
    const SIMDUnrolling<Type> simd_1(1);

    const mwSize num_elems_unroll_8 = simd_8.get_unroll(num_elems);
    const mwSize num_elems_unroll_1 = simd_1.get_unroll(num_elems);

    const AccessType access;

  #pragma omp for nowait
    for(mwIndex elemIdx = 0; elemIdx < num_elems_unroll_8; elemIdx += simd_8.block)
    {
      const Type * const in1 = in_data1 + elemIdx;
      const Type * const in2 = in_data2 + elemIdx;
      const Type * const in3 = in_data3 + elemIdx;
      const Type * const in4 = in_data4 + elemIdx;
      const Type * const in5 = in_data5 + elemIdx;
      Type * const out = out_data + elemIdx;

      APPLY_FUNC_5FOLD_DUAL_DETECTOR(0);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(1);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(2);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(3);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(4);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(5);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(6);
      APPLY_FUNC_5FOLD_DUAL_DETECTOR(7);
    }
  #pragma omp for nowait
    for(mwIndex elemIdx = num_elems_unroll_8; elemIdx < num_elems_unroll_1; elemIdx += simd_1.block)
    {
      const vVvf inV11 = access.load(&in_data1[elemIdx]);
      const vVvf inV21 = access.load(&in_data2[elemIdx]);
      const vVvf inV31 = access.load(&in_data3[elemIdx]);
      const vVvf inV41 = access.load(&in_data4[elemIdx]);
      const vVvf inV51 = access.load(&in_data5[elemIdx]);

      access.store(&out_data[elemIdx], func(inV11, inV21, inV31, inV41, inV51));
    }
  #pragma omp for nowait
    for(mwIndex elemIdx = num_elems_unroll_1; elemIdx < num_elems; elemIdx++)
    {
      out_data[elemIdx] = func(in_data1[elemIdx], in_data2[elemIdx], in_data3[elemIdx], in_data4[elemIdx], in_data5[elemIdx]);
    }
  }

  template<typename Type, typename Function>
  inline void
  dual_detector_iteration(mxArray * outArray,
      const mxArray * const cells_dual,
      const mxArray * const cells_blobs,
      const mxArray * const cells_computed_blobs,
      const mxArray * const cells_sigma1,
      const mxArray * const cells_sigma1_1,
      const Function & func)
  {
    const mwSize num_cells = mxGetNumberOfElements(cells_dual);

  #pragma omp parallel
    {
      for(mwIndex cell_idx = 0; cell_idx < num_cells; cell_idx++)
      {
        const mxArray * const cell_dual = mxGetCell(cells_dual, cell_idx);
        const mxArray * const cell_blob = mxGetCell(cells_blobs, cell_idx);
        const mxArray * const cell_comp_blob = mxGetCell(cells_computed_blobs, cell_idx);
        const mxArray * const cell_sigma1 = mxGetCell(cells_sigma1, cell_idx);
        const mxArray * const cell_sigma1_1 = mxGetCell(cells_sigma1_1, cell_idx);

        const Type * const data_dual = (const Type *) mxGetData(cell_dual);
        const Type * const data_blob = (const Type *) mxGetData(cell_blob);
        const Type * const data_comp_blob = (const Type *) mxGetData(cell_comp_blob);
        const Type * const data_sigma1 = (const Type *) mxGetData(cell_sigma1);
        const Type * const data_sigma1_1 = (const Type *) mxGetData(cell_sigma1_1);

        const mwSize num_elems = mxGetNumberOfElements(cell_dual);

        mxArray * const out_cell = mxGetCell(outArray, cell_idx);
        Type * const out_data = (Type *) mxGetData(out_cell);

        detector_dual_inner_cycle_sse< Type, const Function, AccessAligned<Type> >(
            out_data, data_dual, data_blob, data_comp_blob, data_sigma1, data_sigma1_1, num_elems, func);
      }
    }
  }
};

#endif /* ZUTIL_CXX_INCLUDE_GT6DUPDATEDUALDETECTOROPS_H_ */
