classdef GtBenchmarks
    methods (Static)
        function printHeader(funcName, dataSize)
            fprintf('\n%- 40s (Data size: %f GB)\n', funcName, dataSize / 2^30);
        end

        function printResult(message, result, dataVols)
            fprintf([message '\t']);
            if (exist('dataVols', 'var'))
                fprintf('Throughput: %f GB/s (Time: %f s)\n', dataVols / 2^30 / result, result);
            end
        end

        function bytes = getSizeVariable(var)
            if (iscell(var))
                bytes = 0;
                for ii_c = 1:numel(var)
                    bytes = bytes + GtBenchmarks.getSizeVariable(var{ii_c});
                end
            elseif (isobject(var))
                bytes = 0;
                fnames = fieldnames(var);
                for ii_fn = 1:numel(fnames)
                    bytes = bytes + GtBenchmarks.getSizeVariable(var.(fnames{ii_fn}));
                end
            else
                info = whos('var');
                bytes = info.bytes;
            end
        end

        function verifyError(reference, computed)
            fprintf('Checking errors in computation:\n')
            if (iscell(reference))
                fprintf('\t%g\n', max(cellfun(@(x, y)max(max(max(abs(x - y)))), ...
                    reshape(reference, [numel(reference), 1]), reshape(computed, [numel(computed), 1]) )) );
            elseif (isscalar(reference))
                fprintf('\t%g (%g relative)\n', abs(reference - computed), abs((reference - computed)/reference) );
            else
                fprintf('\t%g\n', max(max(max(abs(reference - computed)))) );
            end
        end
    end
end
