function gt6DAnalyseVoxelProperties(test_data, result, post_result, voxel, threshold_min, threshold_max)
    bigSize = size(result.solution{1});
    finalSize = size(result.deviation{1});
    offsets = (bigSize - finalSize) /2;

    theo_voxel = voxel - offsets;
    fprintf('Distance of reconstructed from real: %d (deg)\n', post_result.distance_deg_alt(theo_voxel(1), theo_voxel(2), theo_voxel(3)))

    intensities_recon = cellfun(@(x)x(voxel(1), voxel(2), voxel(3)), result.solution);

    if (~exist('threshold_min', 'var') || isempty(threshold_min))
        threshold_min = 0;
    end

    if (~exist('threshold_max', 'var') || isempty(threshold_max))
        sorted_intensities = sort(intensities_recon, 'descend');
        index_thresh = min(numel(sorted_intensities), 4);
        threshold_max = sorted_intensities(index_thresh);
        threshold_max = threshold_max - eps('single');
        threshold_max = max(threshold_max, 0);
    end

    gr = [result.grains{:}];
    sampled_R_vecs = cat(1, gr.R_vector);

    f = figure('Name', sprintf('Orientation-space at: (%d, %d, %d), intensity: %g', voxel, sum(intensities_recon)) );
    plot_for_figure = true;
    if (~plot_for_figure)
        ax2 = subplot(1, 4, 1, 'parent', f);
        plot(ax2, intensities_recon);

        ax1 = subplot(1, 4, [2 3 4], 'parent', f);
    else
        ax1 = axes('parent', f);

        set(f, 'Units', 'centimeters')
        set(ax1, 'Units', 'centimeters')

        set(f, 'Position', [0 0 20 14])
        set(f, 'Paperposition', [0 0 20 14])

        set(ax1, 'Position', [1.25 1 16.25 12])
    end
    scatter3(ax1, [], [], []);

    hold(ax1, 'on');

    base_grid = true;
    show_bbox(ax1, sampled_R_vecs, base_grid);

%     % Show all the lattice
%     scatter3(ax, sampled_R_vecs(:, 1), sampled_R_vecs(:, 2), sampled_R_vecs(:, 3), 30, 'm');

    theo_R_vec = show_theoretical_orientation(ax1, test_data.dmvol(theo_voxel(1), theo_voxel(2), theo_voxel(3), :));
    show_average_orientation(ax1, sampled_R_vecs, intensities_recon, theo_R_vec);

%     show_orientations_plain_main(ax1, sampled_R_vecs, intensities_recon, threshold_min, threshold_max);
    show_orientation_colormap(ax1, sampled_R_vecs, intensities_recon, jet(), threshold_min);

    hold(ax1, 'off');
end

function r_vec = show_theoretical_orientation(ax, voxel_theo_Rvec)
    r_vec = permute(voxel_theo_Rvec, [1 4 2 3]);
    scatter3(ax, r_vec(1), r_vec(2), r_vec(3), 30, 'm', 'filled');
end

function show_average_orientation(ax, sampled_R_vecs, intensities_recon, theo_R_vec)
    size(sampled_R_vecs)
    size(intensities_recon)
    size(theo_R_vec)
    avg_R_vec = sum(sampled_R_vecs .* intensities_recon(:, [1 1 1]), 1) ./ sum(intensities_recon);
    scatter3(ax, avg_R_vec(1), avg_R_vec(2), avg_R_vec(3), 30, [0 0 0], 'filled');

    quiv_length = avg_R_vec - theo_R_vec;
    quiver3(ax, theo_R_vec(1), theo_R_vec(2), theo_R_vec(3), quiv_length(1), quiv_length(2), quiv_length(3))
end

function show_orientations_plain_main(ax, sampled_R_vecs, intensities, thr_min, thr_max)
    indexes_recon_min = intensities > thr_min;
    indexes_recon_max = intensities > thr_max;

    voxel_R_vecs = sampled_R_vecs(indexes_recon_min, :);
%     voxel_intensities = intensities(indexes_recon_min);

    main_voxel_R_vecs = sampled_R_vecs(indexes_recon_max, :);

    scatter3(ax, voxel_R_vecs(:, 1), voxel_R_vecs(:, 2), voxel_R_vecs(:, 3), 30, 'b');
    scatter3(ax, main_voxel_R_vecs(:, 1), main_voxel_R_vecs(:, 2), main_voxel_R_vecs(:, 3), 30, 'g', 'filled');
end

function show_orientation_colormap(ax, sampled_R_vecs, intensities, cmap, thr_min)
%     cmap = [flipud(cmap); 0 0 0];

    indexes_recon_min = intensities > thr_min;
    voxel_R_vecs = sampled_R_vecs(indexes_recon_min, :);
    voxel_intensities = intensities(indexes_recon_min);

    min_int = min(voxel_intensities);
    max_int = max(voxel_intensities);

    % Rescaling
    voxel_intensities = (voxel_intensities - min_int) / (max_int - min_int);
    voxel_intensities = voxel_intensities * (size(cmap, 1)-1) + 1;
    voxel_intensities = round(voxel_intensities);

    scatter3(ax, voxel_R_vecs(:, 1), voxel_R_vecs(:, 2), voxel_R_vecs(:, 3), 20, cmap(voxel_intensities, :), 'filled')
    cb = colorbar('peer', ax, 'YTickLabel', min_int:((max_int-min_int)/10):max_int, 'location', 'EastOutside');
    set(cb, 'Units', 'centimeters')
    set(cb, 'Position', [17.35 1 0.65 12])
end

function show_bbox(ax, sampled_R_vecs, base_grid)
    min_sampled_R_vecs = min(sampled_R_vecs, [], 1);
    max_sampled_R_vecs = max(sampled_R_vecs, [], 1);
    bbox_R_vecs = [ ...
        min_sampled_R_vecs(1), min_sampled_R_vecs(2), min_sampled_R_vecs(3); ...
        min_sampled_R_vecs(1), min_sampled_R_vecs(2), max_sampled_R_vecs(3); ...
        min_sampled_R_vecs(1), max_sampled_R_vecs(2), min_sampled_R_vecs(3); ...
        min_sampled_R_vecs(1), max_sampled_R_vecs(2), max_sampled_R_vecs(3); ...

        max_sampled_R_vecs(1), min_sampled_R_vecs(2), min_sampled_R_vecs(3); ...
        max_sampled_R_vecs(1), min_sampled_R_vecs(2), max_sampled_R_vecs(3); ...
        max_sampled_R_vecs(1), max_sampled_R_vecs(2), min_sampled_R_vecs(3); ...
        max_sampled_R_vecs(1), max_sampled_R_vecs(2), max_sampled_R_vecs(3); ...
        ];
    faces = [ ...
        1 5; 2 6; 3 7; 4 8; ...
        1 3; 2 4; 5 7; 6 8; ...
        1 2; 3 4; 5 6; 7 8; ...
        ];

    scatter3(ax, bbox_R_vecs(:, 1), bbox_R_vecs(:, 2), bbox_R_vecs(:, 3), 30, 'y', 'filled');
    patch('parent', ax, 'Faces', faces, 'Vertices', bbox_R_vecs, 'FaceColor', 'w');

    if (base_grid)
        scatter3(ax, sampled_R_vecs(:, 1), sampled_R_vecs(:, 2), sampled_R_vecs(:, 3), 20, 'r');
    end
end