classdef GtOrientationResultView < GtBaseGuiElem
    properties
        test_data;
        result;
        post_result;

        distance_viewer = [];
        distance_comp_viewer = [];
        domains_viewer = [];
        profile_viewer = [];
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public object API - Interaction with outside
    methods (Access = public)
        function self = GtOrientationResultView(test_data, result, post_result, varargin)
            self = self@GtBaseGuiElem();

            self.test_data = test_data;
            self.result = result;
            self.post_result = post_result;

            self.conf.f_title = 'Result Browser';

            self.initGtBaseGuiElem(varargin);
        end

        function display_distance(self)
            if (~isempty(self.distance_viewer) && isvalid(self.distance_viewer))
                figure(self.distance_viewer.conf.h_figure)
            else
                self.distance_viewer = GtVolView(self.post_result.distance_deg_alt);

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Make figure...', ...
                        'Callback', @(evt,src)self.make_distance_figure());

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.distance_viewer));

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Plot 3D Volume from here...', ...
                        'Callback', @(evt,src)self.make_distance_3dplot());
            end
        end

        function display_distance_comp(self)
            if (~isempty(self.distance_comp_viewer) && isvalid(self.distance_comp_viewer))
                figure(self.distance_comp_viewer.conf.h_figure)
            else
                self.distance_comp_viewer = GtMultiVolView(self.post_result.distance_comp_deg);

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Make figure...', ...
                        'Callback', @(evt,src)self.make_distance_comp_figure());

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.distance_comp_viewer));

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Plot 3D Volume from here...', ...
                        'Callback', @(evt,src)self.make_distance_comp_3dplot());
            end
        end

        function display_domains(self)
            if (~isempty(self.domains_viewer) && all(isvalid(self.domains_viewer)))
                for ii = 1:numel(self.domains_viewer)
                    figure(self.domains_viewer(ii).conf.h_figure)
                end
            else
                if (~isempty(self.domains_viewer))
                    % Let's destroy the valid which is left
                    for ii = find(isvalid(self.domains_viewer))'
                        delete(self.domains_viewer(ii))
                    end
                end

                range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];
                picture = {self.post_result.domains_theo, self.post_result.domains_recon};

                self.domains_viewer = GtVolView.compareVolumes(picture, 'clim', range, 'cmap', jet);

                for ii = 1:numel(self.domains_viewer)
                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Make figure...', ...
                            'Callback', @(evt,src)self.make_domains_figure(ii));

                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Analyse Pixel...', ...
                            'Callback', @(evt,src)self.show_pixel_figure(self.domains_viewer(ii)));

                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Plot 3D Volume from here...', ...
                            'Callback', @(evt,src)self.make_domains_3dplot(ii));
                end
            end
        end

        function display_profile(self)
            if (~isempty(self.profile_viewer) && isvalid(self.profile_viewer))
                figure(self.profile_viewer.conf.h_figure)
            else
                self.profile_viewer = GtVolView(self.result.solution);

%                 self.profile_viewer.conf.h_menu_items{end+1} = ...
%                     uimenu(self.profile_viewer.conf.h_context_menu, ...
%                         'Label', 'Make figure...', ...
%                         'Callback', @(evt,src)self.make_distance_figure());

                self.profile_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.profile_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.profile_viewer));
            end
        end

        function save_to_file(self)
            rec_options = gtReconstruct6DGetParamenters(self.test_data.parameters);
            filename = sprintf( ...
                'result-%s-%d_orient-%d-%d_oss-%d_rss-%d.mat', ...
                rec_options.algorithm, rec_options.num_iter, ...
                numel(self.result.grains), rec_options.num_interp, ...
                rec_options.ospace_super_sampling, ...
                rec_options.rspace_super_sampling );

            filter = {'*.mat', 'MATLAB Files (*.mat)'};
            [filename, pathname, filerIndex] = uiputfile( ...
                filter, 'Select file...', filename);

            if (filename)
                result = self.result; %#ok<PROP,NASGU>
                post_result = self.post_result; %#ok<PROP,NASGU>
                save(fullfile(pathname, filename), 'result', 'post_result', 'rec_options', '-v7.3');
            end
        end

        function make_shape_figure(self)
            slice_ind = get(self.conf.h_shape_slice_edit, 'String');
            slice_ind = eval(slice_ind);

            slice = self.result.ODF6D.intensity(:, :, slice_ind);

            extras = gtFigureCreateExtras('box', 'position', [11.5 11.5 50 50], 'color', [1 0 0]);

            gtMakeFigureFromPicture(slice, [], 'cmap', gray, ...
                'colorbar', false, 'borders', false, 'pixel_to_cm', 0.2, ...
                'extras', {extras}, ...
                'pixel_size', self.test_data.parameters.recgeo(1).voxsize);
        end

        function make_orientation_figure(self)
            pos = get(self.conf.h_orient_pos_edit, 'String');
            pos = eval(pos);
            self.make_domains_3dplot(pos)
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Internal methods - Addressable from derived classes
    methods (Access = protected)
        function initGui(obj)
            initGui@GtBaseGuiElem(obj);
            % Let's disable zoom, while configuring
            % must disable zoom before setting impixelinfoval (R2008A)
            zoomOp = zoom(obj.conf.h_figure);
            zoomState = get(zoomOp, 'Enable');
            zoom(obj.conf.h_figure, 'off');

            obj.conf.main_boxes = uiextras.HBox('Parent', obj.conf.currentParent);

            % Layout for buttons used to open windows
            obj.conf.button_boxes = uiextras.VButtonBox('Parent', obj.conf.main_boxes);
            set(obj.conf.button_boxes, 'ButtonSize', [200, 30])

            uicontrol('Parent', obj.conf.button_boxes, 'Style', 'text', ...
                'String', 'Windows:');

            obj.conf.h_profile_button = uicontrol( ...
                'Parent', obj.conf.button_boxes, 'Style', 'pushbutton', ...
                'String', 'Grain Profile');
            obj.conf.h_distance_button = uicontrol( ...
                'Parent', obj.conf.button_boxes, 'Style', 'pushbutton', ...
                'String', 'Distance (deg)');
            obj.conf.h_distance_comp_button = uicontrol( ...
                'Parent', obj.conf.button_boxes, 'Style', 'pushbutton', ...
                'String', 'Distance Components (deg)');
            obj.conf.h_domains_button = uicontrol( ...
                'Parent', obj.conf.button_boxes, 'Style', 'pushbutton', ...
                'String', 'Domains');

            % Spacer
            uicontrol('Parent', obj.conf.button_boxes, 'Enable', 'off', ...
                'Visible', 'off', 'Tag', 'spacer');

            uicontrol('Parent', obj.conf.button_boxes, 'Style', 'text', ...
                'String', 'Saving:');

            obj.conf.h_save_button = uicontrol(...
                'Parent', obj.conf.button_boxes, 'Style', 'pushbutton', ...
                'String', 'Save to file');

            % Layout for buttons used to create figures
            obj.conf.figures_boxes = uiextras.VBox('Parent', obj.conf.main_boxes);
            % Spacer
            uicontrol('Parent', obj.conf.figures_boxes, 'Enable', 'off', ...
                'Visible', 'off', 'Tag', 'spacer');

            uicontrol('Parent', obj.conf.figures_boxes, 'Style', 'text', ...
                'String', 'Figures:');

            % Shape figures
            obj.conf.shape_boxes = uiextras.HButtonBox('Parent', obj.conf.figures_boxes);
            set(obj.conf.shape_boxes, 'ButtonSize', [120, 30])

            obj.conf.h_shape_button = uicontrol( ...
                'Parent', obj.conf.shape_boxes, 'Style', 'pushbutton', ...
                'String', 'Shape figure');

            uicontrol('Parent', obj.conf.shape_boxes, 'Style', 'text', ...
                'String', 'Slice:');

            obj.conf.h_shape_slice_edit = uicontrol('Style', 'edit', ...
                'Parent', obj.conf.shape_boxes, 'String', '40');

            % Orientation figures
            obj.conf.orient_boxes = uiextras.HButtonBox('Parent', obj.conf.figures_boxes);
            set(obj.conf.orient_boxes, 'ButtonSize', [120, 30])

            obj.conf.h_orient_button = uicontrol( ...
                'Parent', obj.conf.orient_boxes, 'Style', 'pushbutton', ...
                'String', 'Orientation figure');

            uicontrol('Parent', obj.conf.orient_boxes, 'Style', 'text', ...
                'String', 'Position:');

            obj.conf.h_orient_pos_edit = uicontrol('Style', 'edit', ...
                'Parent', obj.conf.orient_boxes, 'String', '[42 42 8]');

            % Spacer
            uicontrol('Parent', obj.conf.figures_boxes, 'Enable', 'off', ...
                'Visible', 'off', 'Tag', 'spacer');
            set(obj.conf.figures_boxes, 'Sizes', [-1, 30, 35, 35, -1])

            % Reset back the zoom to the previous state (before the inclusion)
            set(zoomOp, 'Enable', zoomState);
        end

        function addUICallbacks(self)
            % windows buttons
            self.addUICallback(self.conf.h_profile_button, 'Callback', ...
                @(src, evt)display_profile(self), false);

            self.addUICallback(self.conf.h_distance_button, 'Callback', ...
                @(src, evt)display_distance(self), false);

            self.addUICallback(self.conf.h_distance_comp_button, 'Callback', ...
                @(src, evt)display_distance_comp(self), false);

            self.addUICallback(self.conf.h_domains_button, 'Callback', ...
                @(src, evt)display_domains(self), false);

            % Saving button
            self.addUICallback(self.conf.h_save_button, 'Callback', ...
                @(src, evt)save_to_file(self), false);

            % Figures buttons
            self.addUICallback(self.conf.h_shape_button, 'Callback', ...
                @(src, evt)make_shape_figure(self), false);

            self.addUICallback(self.conf.h_orient_button, 'Callback', ...
                @(src, evt)make_orientation_figure(self), false);
        end

        function resetUiComponents(self)
        end

        function doUpdateDisplay(self)
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public interaction API - Interaction with called viewers
    methods (Access = public)
        function make_distance_figure(self)
            point = self.distance_viewer.conf.clicked_point;
            point = self.distance_viewer.getComplete3DPixelInfo(point);
            plane = self.distance_viewer.conf.plane;
            layer = point(self.distance_viewer.conf.ortDim);

            gt6DMakeFigureForPaper(self.post_result.distance_deg_alt, ...
                'colorbar', 'yes', 'border', 'yes', 'filename', '', ...
                'layer', layer, 'plane', plane)
            fprintf([ ...
                'gt6DMakeFigureForPaper(post_result.distance_deg_alt, ' ...
                    '''colorbar'', ''yes'', ''border'', ''yes'', ' ...
                    '''filename'', '''', ''layer'', %d, ''plane'', ''%s'')\n'], ...
                layer, plane)
        end

        function make_distance_comp_figure(self)
            point = self.distance_comp_viewer.conf.clicked_point;
            point = self.distance_comp_viewer.getComplete3DPixelInfo(point);
            plane = self.distance_comp_viewer.conf.plane;
            layer = point(self.distance_comp_viewer.conf.ortDim);
            component = self.distance_comp_viewer.conf.volumeIndex;

            gt6DMakeFigureForPaper(self.post_result.distance_comp_deg(:, :, :, component), ...
                'colorbar', 'yes', 'border', 'yes', 'filename', '', ...
                'layer', layer, 'plane', plane)
            fprintf([ ...
                'gt6DMakeFigureForPaper(post_result.distance_comp_deg(:, :, :, component), ' ...
                    '''colorbar'', ''yes'', ''border'', ''yes'', ' ...
                    '''filename'', '''', ''layer'', %d, ''plane'', ''%s'')\n'], ...
                layer, plane)
        end

        function make_domains_figure(self, indx)
            point = self.domains_viewer(indx).conf.clicked_point;
            point = self.domains_viewer(indx).getComplete3DPixelInfo(point);
            plane = self.domains_viewer(indx).conf.plane;
            layer = point(self.domains_viewer(indx).conf.ortDim);

            range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];

            gt6DMakeFigureForPaper(self.post_result.domains_theo, 'clim', range, 'layer', layer, 'plane', plane)
            gt6DMakeFigureForPaper(self.post_result.domains_recon, 'clim', range, 'layer', layer, 'plane', plane)
            fprintf([ ...
                'gt6DMakeFigureForPaper(post_result.domains_recon, ' ...
                    '''clim'', [%d %d], ' ...
                    '''layer'', %d, ''plane'', ''%s'')\n'], ...
                range, layer, plane)
        end

        function make_distance_3dplot(self)
            point = self.distance_viewer.conf.clicked_point;
            point = self.distance_viewer.getComplete3DPixelInfo(point);
            gt6DPlotVolumeSlices(self.post_result.distance_deg_alt, point, 'colorbar', 'yes');
        end

        function make_distance_comp_3dplot(self)
            point = self.distance_comp_viewer.conf.clicked_point;
            point = self.distance_comp_viewer.getComplete3DPixelInfo(point);
            component = self.distance_comp_viewer.conf.volumeIndex;
            gt6DPlotVolumeSlices(self.post_result.distance_comp_deg(:, :, :, component), point, 'colorbar', 'yes');
        end

        function make_domains_3dplot(self, indx)
            if (numel(indx) == 1)
                point = self.domains_viewer(indx).conf.clicked_point;
                point = self.domains_viewer(indx).getComplete3DPixelInfo(point);
            else
                point = indx;
            end

            range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];

            gt6DPlotVolumeSlices(self.post_result.domains_theo, point, 'clim', range)
            gt6DPlotVolumeSlices(self.post_result.domains_recon, point, 'clim', range)
        end

        function show_pixel_figure(self, viewer)
            point = viewer.conf.clicked_point;
            point = viewer.getComplete3DPixelInfo(point);
            gt6DAnalyseVoxelProperties(self.test_data, self.result, self.post_result, point);
        end
    end
end