function gtFigureAddExtras(ax, im_props, extras)
% function gtFigureAddExtras(ax, im_props, extras)

    num_extras = numel(extras);

    for ii_ex = 1:num_extras
        switch (extras{ii_ex}.type)
            case 'unit_bar'
                add_unit_bar(ax, im_props, extras{ii_ex});
            case {'line_vertical', 'line_horizontal'}
                add_line(ax, im_props, extras{ii_ex})
            case 'box'
                add_box(ax, extras{ii_ex})
            case 'text'
                add_text(ax, extras{ii_ex})
            case 'arrow'
                add_arrow(ax, extras{ii_ex})
            case 'segment'
                add_segment(ax, extras{ii_ex})
        end
    end
end

function add_unit_bar(ax, im_props, ex_props)
    bar_length = ex_props.line_length / im_props.pixel_size(2);

    switch (ex_props.position)
        case 'north_west'
            xs_l = ex_props.margin([1 1]) + [0 bar_length];
            ys_l = ex_props.margin([2 2]);
            x_t = ex_props.margin(1);
            y_t = ex_props.margin(2) + 1; % + ex_props.line_width
            font_placing_vert = 'top';
            font_placing_horz = 'left';
        case 'south_west'
            xs_l = ex_props.margin([1 1]) + [0 bar_length];
            ys_l = im_props.size([2 2]) - ex_props.margin([2 2]);
            x_t = ex_props.margin(1);
            y_t = im_props.size(2) - ex_props.margin(2) - 1; % - ex_props.line_width
            font_placing_vert = 'bottom';
            font_placing_horz = 'left';
        case 'north_est'
            xs_l = im_props.size([1 1]) - ex_props.margin([1 1]) - [0 bar_length];
            ys_l = ex_props.margin([2 2]);
            x_t = im_props.size(1) - ex_props.margin(1); % - bar_length
            y_t = ex_props.margin(2) + 1; % + ex_props.line_width
            font_placing_vert = 'top';
            font_placing_horz = 'right';
        case 'south_est'
            xs_l = im_props.size([1 1]) - ex_props.margin([1 1]) - [0 bar_length];
            ys_l = im_props.size([2 2]) - ex_props.margin([2 2]);
            x_t = im_props.size(1) - ex_props.margin(1); % - bar_length
            y_t = im_props.size(2) - ex_props.margin(2) - 1; % - ex_props.line_width
            font_placing_vert = 'bottom';
            font_placing_horz = 'right';
    end

    line(xs_l, ys_l, 'parent', ax, 'Color', ex_props.color, 'LineWidth', ex_props.line_width)
    text(x_t, y_t, sprintf('%g %s', ex_props.line_length, ex_props.base_unit), ...
        'parent', ax, 'Color', ex_props.color,  ...
        'FontSize', ex_props.font_size, ...
        'FontAngle', ex_props.font_angle, ...
        'FontWeight', ex_props.font_weight, ...
        'VerticalAlignment', font_placing_vert, ...
        'HorizontalAlignment', font_placing_horz )
end

function add_line(ax, im_props, ex_props)

    switch (ex_props.type)
        case 'line_vertical'
            xs_l = ex_props.position([1 1]);
            ys_l = [1 im_props.size(2)];
        case 'line_horizontal'
            xs_l = [1 im_props.size(1)];
            ys_l = ex_props.position([1 1]);
    end

    line(xs_l, ys_l, 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
end

function add_box(ax, ex_props)

    x_nw = ex_props.position(1);
    y_nw = ex_props.position(2);

    x_sw = ex_props.position(1);
    y_sw = ex_props.position(2) + ex_props.position(4) - 1;

    x_ne = ex_props.position(1) + ex_props.position(3) - 1;
    y_ne = ex_props.position(2);

    x_se = ex_props.position(1) + ex_props.position(3) - 1;
    y_se = ex_props.position(2) + ex_props.position(4) - 1;

    line([x_nw x_sw], [y_nw y_sw], 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
    line([x_ne x_se], [y_ne y_se], 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
    line([x_nw x_ne], [y_nw y_ne], 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
    line([x_sw x_se], [y_sw y_se], 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
end

function add_text(ax, ex_props)
    text(ex_props.position(1), ex_props.position(2), ex_props.string, ...
        'parent', ax, 'Color', ex_props.color, ...
        'FontSize', ex_props.font_size, ...
        'FontAngle', ex_props.font_angle, ...
        'FontWeight', ex_props.font_weight, ...
        'HorizontalAlignment', ex_props.alignment_horz, ...
        'VerticalAlignment', ex_props.alignment_vert, ...
        'Interpreter', ex_props.interpreter );
end

function add_arrow(ax, ex_props)

    final_dir = ex_props.direction * ex_props.line_length;

    hold(ax, 'on')
    quiver(ex_props.position(1), ex_props.position(2), final_dir(1), final_dir(2), 0, ...
        'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
    hold(ax, 'off')
end

function add_segment(ax, ex_props)

    xs_l = ex_props.position([1 3]);
    ys_l = ex_props.position([2 4]);

    line(xs_l, ys_l, 'parent', ax, 'Color', ex_props.color, ...
        'LineWidth', ex_props.line_width, 'LineStyle', ex_props.line_style)
end

