function [f, ax] = gtFigureCreateFromPicture(img, mask, varargin)

    conf = struct( ...
        'filename', '', ...
        'lims_u', [], ...
        'lims_v', [], ...
        'flip_ud', false, ...
        'flip_lr', false, ...
        'transpose', true, ...
        'label', [], ...
        'clims', [], ...
        'cmap', jet, ...
        'pixel_to_cm', 0.05, ...
        'borders', true, ...
        'colorbar', true, ...
        'pixel_size', [], ...
        'extras', [] );
    [conf, rej_conf] = parse_pv_pairs(conf, varargin);
    if (~isempty(rej_conf))
        disp('Unknown parameters:')
        disp(rej_conf)
    end

    use_mask = exist('mask', 'var') && ~isempty(mask);
    if (~use_mask)
        mask = true(size(img, 1), size(img, 2));
    end

    if (~isempty(conf.lims_u))
        mask([1:conf.lims_u(1)-1, conf.lims_u(2)+1:end], :) = 0;
    end
    if (~isempty(conf.lims_v))
        mask(:, [1:conf.lims_v(1)-1, conf.lims_v(2)+1:end]) = 0;
    end

    % cuting the images
    horz_profile = sum(mask, 1);
    vert_profile = sum(mask, 2);
    horz_lims = [find(horz_profile, 1, 'first'), find(horz_profile, 1, 'last')];
    vert_lims = [find(vert_profile, 1, 'first'), find(vert_profile, 1, 'last')];

    img = img(vert_lims(1):vert_lims(2), horz_lims(1):horz_lims(2), :);
    mask = mask(vert_lims(1):vert_lims(2), horz_lims(1):horz_lims(2));

    % Note the inversion between lr/up because of matlab's transpose problem
    if (conf.flip_ud)
        img = fliplr(img);
        mask = fliplr(mask);
    end
    if (conf.flip_lr)
        img = flipud(img);
        mask = flipud(mask);
    end
    if (conf.transpose)
        img = permute(img, [2 1 3]);
        mask = permute(mask, [2 1 3]);
    end

    if (isempty(conf.clims))
        if (size(img, 3) == 3)
            clims_img = [0 1];
        else
            clims_img = [min(img(:)), max(img(:))];
        end
    else
        clims_img = [conf.clims(1), min(conf.clims(2), max(img(:)))];
    end

    img_size = [size(final_slice_RGB, 2), size(final_slice_RGB, 1)];

    f = figure();
    ax = axes('parent', f);

    if (use_mask)
        imagesc(img, 'parent', ax, ...
            'AlphaDataMapping', 'scaled', 'AlphaData', mask', ...
            clims_img );
    else
        imagesc(img, 'parent', ax, ...
            clims_img );
    end
    colormap(ax, conf.cmap)

    if (~isempty(conf.extras))
        if (isempty(conf.pixel_size))
            p = gtLoadParameters();
            conf.pixel_size = p.recgeo.voxsize * 1e3;
        end
        im_props = struct( ...
            'pixel_size', conf.pixel_size, ...
            'size', img_size );
        gtFigureAddExtras(ax, im_props, conf.extras);
    end

    gtFigureSetSize(f, ax, conf, img_size);
end
