function gtSetupForwardSimulation()
% GTSETUPFORWARDSIMULATION  Simple helper function to run gtForwardSimulate_v2
%     gtSetupForwardSimulation()
%     --------------------------

parameters = gtLoadParameters();

% Let's check every field is in, and in case let's update it!
parameters = gtCheckParameters(parameters, 'fsim', 'verbose', true);
parameters = gtCheckParameters(parameters, 'recgeo', 'verbose', true);
if (~isfield(parameters, 'detgeo') || isempty(parameters.detgeo))
    parameters.detgeo = gtGeoConvertLegacyLabgeo2Detgeo(parameters.labgeo);
end
if (~isfield(parameters.labgeo, 'omstep') || isempty(parameters.labgeo.omstep))
    parameters.labgeo.omstep = gtGetOmegaStepDeg(parameters);
end
% Updating recgeo with a consistent set of parameters
parameters.recgeo = gtGeoRecDefaultParameters(parameters.detgeo, parameters.samgeo);
gtSaveParameters(parameters);

gtDBConnect();

check = inputwdefault('Do you want to recompile the function ''gtForwardSimulate_v2''? [y/n]', 'n');
if strcmpi(check, 'y')
    gtExternalCompileFunctions('gtForwardSimulate_v2');
end

nof_phases = length(parameters.cryst);

if (nof_phases > 1)
    phaseID = inputwdefaultnumeric('Enter phaseID', num2str(1));
else
    phaseID = 1;
end

phaseID_str = sprintf('%02d', phaseID);

disp(['Setting up Forward Simulation for phase ' phaseID_str ' ' parameters.cryst(phaseID).name]);

% get the latest version of indexter output
ngrains = zeros(nof_phases, 1);
for ii = 1:nof_phases
    grain = [];
    index_filename = fullfile(parameters.acq.dir, '4_grains', sprintf('phase_%02d', ii), 'index.mat');
    load(index_filename, 'grain');
    % how many grains in this dataset?
    ngrains(ii) = length(grain);
end

check = inputwdefault('Do you want to reset grainID/flag/phaseID in the difspot table? [y/n]', 'y');
if strcmpi(check, 'y')
    [columns, ~, ~, ~, ~, ~] = mym(['SHOW COLUMNS FROM ' parameters.acq.name 'difspot']);
    if ismember('grainID', columns)
        mym(['UPDATE ' parameters.acq.name 'difspot SET grainID=null']);
        disp(['Variables grainID has been resetted in table ' parameters.acq.name 'difspot'])
    end
    if ismember('phaseID', columns)
        mym(['UPDATE ' parameters.acq.name 'difspot SET phaseID=null']);
        disp(['Variable phaseID has been resetted in table ' parameters.acq.name 'difspot'])
    end
    if ismember('flag', columns)
        mym(['UPDATE ' parameters.acq.name 'difspot SET flag=null']);
        disp(['Variable flag has been resetted in table ' parameters.acq.name 'difspot'])
    end
end

% reset grain_####.mat
for ii = 1:nof_phases
    check = inputwdefault(sprintf('Do you want to delete the previous grain_####.mat files for phase %d? [y/n]', ii), 'y');
    if strcmpi(check, 'y')
        delete( fullfile(parameters.acq.dir, '4_grains', sprintf('phase_%02d', ii), 'grain_*.mat') );
    else
        fprintf('You may have problems in the future with the grain_####.mat files for phase %d.\n', ii);
    end
end

% create / reset sample.mat
sample_filename = fullfile(parameters.acq.dir, '4_grains', 'sample.mat');
if ( exist(sample_filename, 'file') && nof_phases > 1 )
    % do not create from scratch - we want to keep information related to other phases
    sample = GtSample.loadFromFile(sample_filename);
    sample.phases{phaseID} = GtPhase(parameters.cryst(phaseID).name, ngrains(phaseID));
else
    sample = GtSample(nof_phases);
    sample.fileTable = [parameters.acq.name '_filetable'];
    for ii = 1:nof_phases
        sample.phases{ii} = GtPhase(parameters.cryst(ii).name, ngrains(ii));
    end
end
sample.saveToFile(sample_filename);

disp(['Reset phase number ' phaseID_str ' in the sample.mat:']);
disp(['  Phase name : ' parameters.cryst(phaseID).name]);
disp(['  Nof_grains : ' num2str(ngrains(phaseID))]);

% create / reset spot2grain.mat
filename = fullfile(parameters.acq.dir, '4_grains', 'spot2grain.mat');
difspot_num  = mym(['select count(*) from ' parameters.acq.name 'difspot']);

if ( exist(filename, 'file') && nof_phases > 1 )
    check = inputwdefault('Do you want to reset spot2grain? [y/n]', 'n');
    if strcmpi(check, 'y')
        spot2grain  = cell(difspot_num, 1); % cell array linking difspots with grainids
        spot2phase  = cell(difspot_num, 1);
        save(filename, 'spot2grain', 'spot2phase');
    end
else
    spot2grain  = cell(difspot_num, 1); % cell array linking difspots with grainids
    spot2phase  = cell(difspot_num, 1);
    save(filename, 'spot2grain', 'spot2phase');
end

% get reconstruction parameters
check = inputwdefault('Use existing parameters for Forward Simulation? [y/n]', 'y');
if ~strcmpi(check, 'y')
    list = build_list_v2();
    parameters.fsim = gtModifyStructure(parameters.fsim, list.fsim, 1, 'Forward simulation parameters:');
    save(fullfile(parameters.acq.dir, 'parameters.mat'), 'parameters');
end

% launch forward simulation process to OAR - split to two jobs
check = inputwdefault([ ...
    'Launch Forward Simulation with OAR (o), here on the command line ' ...
    '(c) or quit the forward simulation (q)? [o/c/q]'], 'o');
switch(lower(check))
    case 'o'
        disp('Launching several separate OAR jobs');

        oar.njobs    = min(ceil(ngrains(phaseID)/20), 30); % max 30 jobs
        oar.walltime = ngrains(phaseID) / oar.njobs * 5 * 60;  % allow 5 minutes per grain % in seconds
        oar.mem      = 2000; %Mb

        % oar parameters are defined in build_list_v2
        list = build_list_v2();
        OAR_parameters = gtModifyStructure(oar, list.oar, 1, 'OAR parameters:');

        OARtmp = ceil(OAR_parameters.walltime / 60); % minutes
        OARm   = mod(OARtmp, 60); % minutes
        OARh   = (OARtmp - OARm) / 60; % hours

        OAR_parameters.walltime = sprintf('%02d:%02d:00', OARh, OARm);

        disp('Using OAR parameters:')
        disp(OAR_parameters)
        disp(['Using ' num2str(OAR_parameters.njobs) ' OAR jobs']);

        pause(0.5);

        if (isfield(parameters.labgeo, 'sourcepoint') ...
                && ~isempty(parameters.labgeo.sourcepoint))
            OAR_make('gtForwardSimulatePoly', 1, ngrains(phaseID), OAR_parameters.njobs, ...
                [parameters.acq.dir ' ' phaseID_str], true, ...
                'walltime', OAR_parameters.walltime, 'mem', OAR_parameters.mem);
        else
            OAR_make('gtForwardSimulate_v2', 1, ngrains(phaseID), OAR_parameters.njobs, ...
                [parameters.acq.dir ' ' phaseID_str], true, ...
                'walltime', OAR_parameters.walltime, 'mem', OAR_parameters.mem);
        end
    case 'c'
        disp('Launching forward simulation...')
        disp(' ')

        if isfield(parameters.labgeo, 'sourcepoint') && ~isempty(parameters.labgeo.sourcepoint)
            gtForwardSimulatePoly(1, ngrains(phaseID), parameters.acq.dir, phaseID);
        else
            gtForwardSimulate_v2(1, ngrains(phaseID), parameters.acq.dir, phaseID, 'display_figure', false);
        end
    otherwise
        disp('Quitting forward simulation...')
        return
end

end % end of function

function vars_out = sfStruct2String(vars_in)
    vars_out = {};
    % transform logical and numerical into strings
    vars_out(end+1:end+length(fieldnames(vars_in))*2) = struct2pv(vars_in);
    ind = find(cellfun(@(x) isempty(x), vars_out(2:2:end)));
    vars_out([ind*2-1, ind*2]) = [];

    ind = find(cellfun(@(x) isnumeric(x), vars_out));
    vars_out(ind) = cellfun(@(x) num2str(x), vars_out(ind), 'UniformOutput', false);
    ind = find(cellfun(@(x) islogical(x), vars_out));
    vars_out(ind) = cellfun(@(x) num2str(x), vars_out(ind), 'UniformOutput', false);
    ind = find(cellfun(@(x) isvector(x), vars_out));
    vars_out(ind) = cellfun(@(x) strrep(num2str(x), ' ', ''), vars_out(ind), 'UniformOutput', false);
    vars_out(2, :) = repmat({' '}, 1, length(vars_out));

    vars_out = strtrim([vars_out{:}]);
end
