function par_recgeo = gtGeoRecDefaultParameters(detgeo, samgeo)
% GTGEOSAMDEFAULTPARAMETERS  Stores and returns the default Sample geomtery
%                            parameters
%
%     par_recgeo = gtGeoRecDefaultParameters([detgeo [, samgeo]])
%     ---------------------------------------
%     The default Reconstruction reference is set as the Lab but rotating with the
%     sample. Voxel size is determined from detector pixelsize.
%

    if (exist('samgeo', 'var') && ~isempty(samgeo))
        par_recgeo.orig = samgeo.orig;
        par_recgeo.dirx = samgeo.dirx;
        par_recgeo.diry = samgeo.diry;
        par_recgeo.dirz = samgeo.dirz;
    else
        par_recgeo.orig = [0 0 0];
        par_recgeo.dirx = [1 0 0];
        par_recgeo.diry = [0 1 0];
        par_recgeo.dirz = [0 0 1];
    end

    if (exist('detgeo', 'var') && ~isempty(detgeo))
        det_pixel_size = mean([detgeo.pixelsizeu, detgeo.pixelsizev]);
    else
        det_pixel_size = 1e-3;
    end

    par_recgeo.voxsize = det_pixel_size([1 1 1]);
end
