function [inds, ints] = gtMathsGetInterpolationIndices(pos, pows, data_type)
% FUNCTION [inds, ints] = gtMathsGetInterpolationIndices(pos, pows, data_type)
%

    if (exist('data_type', 'var') && ~isempty(data_type))
        pos = cast(pos, data_type);
        pows = cast(pows, data_type);
    end

    n_dims = size(pos, 2);

    pos_l = floor(pos); % lower position indices
    pos_h = pos_l + 1;  % higher position indices

    mu_l = pos_h - pos;
    mu_h = 1 - mu_l;

    switch (n_dims)
        case 3
            % uvw indices of 8 nearest neighbours ((nx8) x 3)
            inds = [ ...
                [ pos_l(:, 1), pos_l(:, 2), pos_l(:, 3) ]; ...
                [ pos_l(:, 1), pos_l(:, 2), pos_h(:, 3) ]; ...
                [ pos_l(:, 1), pos_h(:, 2), pos_l(:, 3) ]; ...
                [ pos_l(:, 1), pos_h(:, 2), pos_h(:, 3) ]; ...
                [ pos_h(:, 1), pos_l(:, 2), pos_l(:, 3) ]; ...
                [ pos_h(:, 1), pos_l(:, 2), pos_h(:, 3) ]; ...
                [ pos_h(:, 1), pos_h(:, 2), pos_l(:, 3) ]; ...
                [ pos_h(:, 1), pos_h(:, 2), pos_h(:, 3) ] ];
%             inds = [ pos_l; ...
%                 [ pos_l(:, [1, 2]), pos_h(:, 3) ]; ...
%                 [ pos_l(:, 1),      pos_h(:, 2),     pos_l(:, 3) ]; ...
%                 [ pos_l(:, 1),      pos_h(:, [2, 3]) ]; ...
%                 [ pos_h(:, 1),      pos_l(:, [2, 3]) ]; ...
%                 [ pos_h(:, 1),      pos_l(:, 2),     pos_h(:, 3) ]; ...
%                 [ pos_h(:, [1, 2]), pos_l(:, 3) ]; ...
%                 pos_h ];

            ints = [ ...
                mu_l(:, 1) .* mu_l(:, 2) .* mu_l(:, 3) .* pows; ...
                mu_l(:, 1) .* mu_l(:, 2) .* mu_h(:, 3) .* pows; ...
                mu_l(:, 1) .* mu_h(:, 2) .* mu_l(:, 3) .* pows; ...
                mu_l(:, 1) .* mu_h(:, 2) .* mu_h(:, 3) .* pows; ...
                mu_h(:, 1) .* mu_l(:, 2) .* mu_l(:, 3) .* pows; ...
                mu_h(:, 1) .* mu_l(:, 2) .* mu_h(:, 3) .* pows; ...
                mu_h(:, 1) .* mu_h(:, 2) .* mu_l(:, 3) .* pows; ...
                mu_h(:, 1) .* mu_h(:, 2) .* mu_h(:, 3) .* pows ];
        case 2
            inds = [ pos_l; ...
                [ pos_l(:, 1), pos_h(:, 2) ]; ...
                [ pos_h(:, 1), pos_l(:, 2) ]; ...
                pos_h ];

            ints = [ ...
                mu_l(:, 1) .* mu_l(:, 2) .* pows; ...
                mu_l(:, 1) .* mu_h(:, 2) .* pows; ...
                mu_h(:, 1) .* mu_l(:, 2) .* pows; ...
                mu_h(:, 1) .* mu_h(:, 2) .* pows ];
        case 1
            inds = [ pos_l; pos_h ];

            ints = [ mu_l .* pows; mu_h .* pows ];
        otherwise
            error('gtMathsGetInterpolationIndices:wrong_argument', ...
                'Interpolation can only be done for 1D, 2D and 3D positions')
    end
end
