function r = gtMathsOriMat2Rod(g)
% GTMATHSROTMAT2EULER  Convert orientation matrices to Rodrigues vectors.
%
%     r = gtMathsOriMat2Rod(g)
%     -------------------------------------------------------------------------
%
%     INPUT:
%       g = <double>  Orientation matrices (3x3xN array).
%                     See Randle & Engler, 2000 for more details about g.
%
%     OUTPUT:
%       r = <double>  Rodrigues vectors, as column vectors (3xN array).
%
%     Version 002 17-10-2012 by YGuilhem
%       Vectorize function to process quickly lots of orientation matrices
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of g matrices
if (~isreal(g))
    gtError('gtMathsOriMat2Rod:wrong_input_type', ...
        'Wrong input type, wait a 3x3xN real array!');
elseif (size(g, 1)~=3 || size(g, 2)~=3)
    gtError('gtMathsOriMat2Rod:wrong_input_size', ...
        'Input array g should be sized 3x3xN!');
end

% Set epsilon to avoid singular values
epsilon = 1.e-9;

% Compute orientation matrices traces
g_trace_plus1 = g(1, 1, :) + g(2, 2, :) + g(3, 3, :) + 1;

% Find regular cases and prepare multiplier (null for non regular cases)
multiplier = double(abs(g_trace_plus1) >= epsilon)./(g_trace_plus1);

r = [ g(2, 3, :) - g(3, 2, :) ; ...
      g(3, 1, :) - g(1, 3, :) ; ...
      g(1, 2, :) - g(2, 1, :) ];

r = r .* multiplier([1 1 1], :, :);
 
% Reshape resulting matrix 
% r = squeeze(r);
r = reshape(r, 3, []); % Runs faster than squeeze

end % end of function
