function list = build_list_v2()
% BUILD_LIST  Builds a list of {fieldname, description, class, user_type}
%             for each field of parameters.mat
%
%     list = build_list_v2()
%     ----------------------
%
%     OUTPUT:
%       list = <cell> structure with each field (N,4)
%
%
%     Version 006 01-08-2013 by LNervo
%       Added 'opsym' and 'symm' fields to 'cryst' for FABLE calculations of
%       reflections
%       Added OAR part at the end (can it be useful?)
%     Modified on 01-10-2013 LNervo : opsym -> sg
%
%     Version 005 04-10-2012 by LNervo
%       Added 'fsim.save_grain'
%
%     Version 004 12-06-2012 by LNervo
%       Added forth column for user flag: 0:not display, 1:basic, 2:expert
%
%     Version 003 23-05-2012 by LNervo
%       Removed lab2sam and sam2lab from samgeo
%
%     Version 002-1 24/04/2012 by LNervo
%       Formatting and commenting
%
%     Version 002 08-03-2012 by PReischig
%       parameters version 2 (from March 2012)
%         added    : version, labgeo, samgeo, fsim
%         modified : acq, xop, cryst, seg, match, rec
%
%       The list.mat has the same fields as parameters.mat, except for:
%           cryst__sg
%           cryst__symm
%
%           index__strategy
%           index__strategy__b
%           index__strategy__b__beg
%           index__strategy__b__end
%           index__strategy__m
%           index__strategy__m__beg
%           index__strategy__m__end

list = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Version number of parameters file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.version(1, :) = {'number', ...
    'Version number of this parameters structure', 'double', 0};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Acquisition parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.acq = cell(0, 4);
list.acq(end+1, :) = {'collection_dir', ...
    'Collection directory', 'char', 1};
list.acq(end+1, :) = {'name', ...
    'Name of the dataset', 'char', 1};
list.acq(end+1, :) = {'dir', ...
    'Directory in which to analyse the data', 'char', 1};
list.acq(end+1, :) = {'date', ...
    'Date of acquisition', 'char', 0};
list.acq(end+1, :) = {'xdet', ...
    'Detector ROI size X or U (raw image size in pixels)', 'double', 1};
list.acq(end+1, :) = {'ydet', ...
    'Detector ROI size Y or V (raw image size in pixels)', 'double', 1};
list.acq(end+1, :) = {'nproj', ...
    'Number of images in *180 DEGREES* of scan', 'double', 1};
list.acq(end+1, :) = {'refon', ...
    'References after how many images', 'double', 2};
list.acq(end+1, :) = {'nref', ...
    'How many reference images in a group', 'double', 2};
list.acq(end+1, :) = {'ndark', ...
    'How many dark images taken', 'double', 2};
list.acq(end+1, :) = {'pixelsize', ...
    'Detector pixelsize (mm/pixel)', 'double', 1};
list.acq(end+1, :) = {'count_time', ...
    'Image integration time (s)', 'double', 1};
list.acq(end+1, :) = {'energy', ...
    'Beam energy (keV)', 'double', 1};
list.acq(end+1, :) = {'dist', ...
    'Sample-detector distance (mm) [computed]', 'double', 1};
list.acq(end+1, :) = {'sensortype', ...
    'Camera type (''frelon''/''kodak4mv1'')', 'char', 1};
list.acq(end+1, :) = {'type', ...
    'DCT scan type (''360degree'',''180degree'', etc)', 'char', 1};
list.acq(end+1, :) = {'interlaced_turns', ...
    'Interlaced scan? 0 for normal scan, 1 for one extra turn, etc', 'double', 1};
list.acq(end+1, :) = {'mono_tune', ...
    'Monochromator was tuned after N reference groups, or 0 for not tuned', 'double', 1};
list.acq(end+1, :) = {'rotation_axis', ...
    'Rotation axis orientation (''vertical''/''horizontal'')', 'char', 1};
list.acq(end+1, :) = {'distortion', ...
    'Distortion correction file with path (or ''none'')', 'char', 1};
list.acq(end+1, :) = {'beamchroma', ...
    'Beam chromaticity (''mono''/''poly'')', 'char', 2};
list.acq(end+1, :) = {'no_direct_beam', ...
    'Special scan with no direct beam (taper frelon, offset detector)?', 'logical', 1};
list.acq(end+1, :) = {'rotation_direction', ...
    'Horizontal axis scan - rotate images (''clockwise''/''counterclockwise'')', 'char', 2};
list.acq(end+1, :) = {'detector_definition', ...
    'Definition of the detector type (''inline''/''vertical'')', 'char', 1};
list.acq(end+1, :) = {'collection_dir_old', ...
    '[Computed] Old collection directory', 'char', 0};
list.acq(end+1, :) = {'rotu', ...
    'U coordinate of rotation axis projection in image [computed] ??? equals to rotx ???', 'double', 0};
list.acq(end+1, :) = {'rotx', ...
    'V coordinate of rotation axis projection in image [computed] ??? equals to rotu ???', 'double', 0};
list.acq(end+1, :) = {'bb', ...
    'Sample bbox symm. around rot. axis [umin vmin usize vsize]}; reconstructed sample volume has size [bb(3) bb(3) bb(4)]', 'double', 0};
list.acq(end+1, :) = {'bbdir', ...
    'Direct beam bounding box [umin vmin usize vsize]', 'double', 0};
list.acq(end+1, :) = {'nof_phases', ...
    'Number of phases in sample to be analysed', 'double', 1};
list.acq(end+1, :) = {'true_detsizeu', ...
    'True detector X or U size before cropping (pixels)', 'double', 2};
list.acq(end+1, :) = {'true_detsizev', ...
    'True detector Y or V size before cropping (pixels)', 'double', 2};
list.acq(end+1, :) = {'detroi_u_off', ...
    'X or U offset of cropped detector ROI (the corner pixel)', 'double', 2};
list.acq(end+1, :) = {'detroi_v_off', ...
    'Y or V offset of cropped detector ROI (the corner pixel)', 'double', 2};
list.acq(end+1, :) = {'pair_tablename', ...
    'Table name for spot pairs', 'char', 0};
list.acq(end+1, :) = {'calib_tablename', ...
    '[not used] Table name for calibration of spot pairs matching', 'char', 0};
list.acq(end+1, :) = {'flip_images', ...
    'Do you want to flip the images left-right for some reason?', 'logical', 1};
list.acq(end+1, :) = {'maxradius', ...
    '[Computed] Maximum active radius of the detector, for example in case of vignetting', 'double', 0};
list.acq(end+1, :) = {'online', ...
    'Is the analysis online?', 'logical', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% xop for phase 1}; if necessary the same for other phases
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.xop = cell(0, 4);
list.xop(end+1, :) = {'twotheta', ...
    'Twotheta angle (2*Bragg angle)', 'single', 0};
list.xop(end+1, :) = {'dspacing', ...
    'd-spacing', 'single', 0};
list.xop(end+1, :) = {'int', ...
    'Intensity', 'single', 0};
list.xop(end+1, :) = {'formfactor', ...
    'Form factor', 'single', 0};
list.xop(end+1, :) = {'mult', ...
    'Multiplicity', 'int8', 0};
list.xop(end+1, :) = {'hkl', ...
    'hkl Miller indexes', 'int8', 0};
list.xop(end+1, :) = {'filename', ...
    'XOP/Diamond crystallographic file', 'char', 0};
list.xop(end+1, :) = {'xop_dir', ...
    'Directory for the XOP input file', 'char', 0};

                                                       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Crystallography
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data of all phases are stored in the array list.cryst:
%  list.cryst(1), list.cryst(2), etc.
% In case of one phase only, both 'list.cryst(1)' and 'list.cryst' can be
%  used in Matlab to refer to that phase.

list.cryst = cell(0, 4);
list.cryst(end+1, :) = {'name', ...
    'Name of phase to display (e.g. Al, Austenite, Beta_Titanium)', 'char', 1};
list.cryst(end+1, :) = {'composition', ...
    'Chemical composition of phase (e.g BaTiO3)', 'char', 1};
% to distinguish materials from different sources or processing
list.cryst(end+1, :) = {'material', ...
    'Distinctive reference name of sample material (e.g AlLi_July2010_recrystallized)', 'char', 1};
list.cryst(end+1, :) = {'latticepar', ...
    'Lattice parameters [a b c alpha beta gamma] (angstrom, deg)', 'double', 2};
list.cryst(end+1, :) = {'spacegroup', ...
    'Crystallographic spacegroup', 'double', 2};
list.cryst(end+1, :) = {'hermann_mauguin', ...
    '[Computed] Hermann Mauguin short symbol', 'char', 0};
% see wikipedia.org/wiki/Crystal_system
list.cryst(end+1, :) = {'crystal_system', ...
    '[Computed] Crystallographic crystal system', 'char', 0};
% to see difference from crystal system, see wikipedia.org/wiki/Crystal_system
list.cryst(end+1, :) = {'lattice_system', ...
    '[Computed] Crystallographic lattice system', 'char', 0};
list.cryst(end+1, :) = {'opsym', ...
    '[Computed] Symmetry operators read from .dsv/.dat file (Diamond/XOP calculation)', 'cell', 0};
list.cryst(end+1, :) = {'hkl', ...
    '[Computed] List of {hkl} families list to be considered', 'double', 0};
list.cryst(end+1, :) = {'hklsp', ...
    '[Computed] List of signed (specific) {hkl} families to be considered', 'double', 0};
list.cryst(end+1, :) = {'theta', ...
    '[Computed] Bragg angles theta corresponding to hkl and acq. energy', 'double', 0};
list.cryst(end+1, :) = {'thetasp', ...
    '[Computed] Bragg angles theta corresponding to hklsp and acq. energy', 'double', 0};
list.cryst(end+1, :) = {'dspacing', ...
    '[Computed] D-spacings corresponding to hkl', 'double', 0};
list.cryst(end+1, :) = {'dspacingsp', ...
    '[Computed] D-spacings corresponding to hklsp', 'double', 0};
list.cryst(end+1, :) = {'thetatype', ...
    '[Computed] Scalar indices of the hkl families (and theta-s)', 'double', 0};
list.cryst(end+1, :) = {'thetatypesp', ...
    '[Computed] Scalar index of the given hkl family (one of thetatype)', 'double', 0};
list.cryst(end+1, :) = {'int', ...
    '[Computed] Intensity of reflection corresponding to hkl', 'double', 0};
list.cryst(end+1, :) = {'intsp', ...
    '[Computed] Intensity of reflection corresponding to hklsp', 'double', 0};
list.cryst(end+1, :) = {'mult', ...
    '[Computed] Multiplicity of reflections from the given hkl family', 'double', 0};
list.cryst(end+1, :) = {'usedfam', ...
    '[Computed] Used hkl family in matching', 'logical', 0};
list.cryst(end+1, :) = {'symm', ...
    '[Computed] Symmetry operators for the given crystal system', 'struct', 0};

list.cryst__symm = cell(0, 4);
list.cryst__symm(end+1, :) = {'g3', ...
    '[Computed] Set of symmetry operators (3x3)', 'double', 0};
list.cryst__symm(end+1, :) = {'g', ...
    '[Computed] Set of symmetry operators (4x4)', 'double', 0};
list.cryst(end+1, :) = {'formfactor', ...
    '[Computed] Formfactor of the hkl families', 'double', 0};

% list.cryst(end+1, :) = {'sg', '[Computed] Additional information about symmetry operators for the given crystal system', 'struct', 0};

% list.cryst__sg = cell(0, 4);
% list.cryst__sg(end+1, :) = {'rot', '[Computed] Complete list of rotations given a crystal system', 'double', 0};
% list.cryst__sg(end+1, :) = {'trans', '[Computed] Complete list of translations values given a crystal system', 'double', 0};

% list.cryst(end+1, :) = {'symm', '[Computed] Unique symmetry operators for the given crystal system', 'struct', 0};

% list.cryst__symm = cell(0, 4);
% list.cryst__symm(end+1, :) = {'g3', '[Computed] Set of unitary rotation matrices corresponding to the indistinguishable lattice permutations', 'double', 0};
% list.cryst__symm(end+1, :) = {'perm', '[Computed] Set of indistinguishable lattice permutations given a crystal system', 'double', 0};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters to describe the setup geometry
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% coordinates are given in the Lab reference
list.labgeo = cell(0, 4);
list.labgeo(end+1, :) = {'beamdir', ...
    'Beam direction in LAB reference (unit row vector)', 'double', 1};
list.labgeo(end+1, :) = {'rotdir', ...
    'Rotation axis direction in LAB (unit row vector); omega is right-handed rotation', 'double', 1};
list.labgeo(end+1, :) = {'rotpos', ...
    'Rotation axis position (arbitrary point on axis) in LAB', 'double', 1};
list.labgeo(end+1, :) = {'samenvtop', ...
    '[Computed] Distance from rotpos to top of sample envelope along rotdir (signed scalar in lab units)', 'double', 0};
list.labgeo(end+1, :) = {'samenvbot', ...
    '[Computed] Distance from rotpos to bottom of sample envelope along rotdir (signed scalar in lab units)', 'double', 0};
list.labgeo(end+1, :) = {'samenvrad', ...
    '[Computed] Radius of sample envelope (in lab units)', 'double', 0};
list.labgeo(end+1, :) = {'labunit', ...
    'LAB units (default is mm) [for records only]', 'char', 1};
list.labgeo(end+1, :) = {'deflabX', ...
    'Description how Lab X direction was chosen [for records only]', 'char', 1};
list.labgeo(end+1, :) = {'deflabY', ...
    'Description how Lab Y direction was chosen [for records only]', 'char', 1};
list.labgeo(end+1, :) = {'deflabZ', ...
    'Description how Lab Z direction was chosen [for records only]', 'char', 1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters to describe the setup geometry
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% coordinates are given in the Det reference
list.detgeo = cell(0, 4);
list.detgeo(end+1, :) = {'detrefpos', ...
    'Detector reference point (usually center) position in LAB', 'double', 1};
list.detgeo(end+1, :) = {'detrefu', ...
    'Detector reference point U coordinate in pixels', 'double', 1};
list.detgeo(end+1, :) = {'detrefv', ...
    'Detector reference point V coordinate in pixels', 'double', 1};
list.detgeo(end+1, :) = {'detdiru', ...
    'Detector U direction in LAB (unit row vector)', 'double', 1};
list.detgeo(end+1, :) = {'detdirv', ...
    'Detector V direction in LAB (unit row vector)', 'double', 1};
list.detgeo(end+1, :) = {'pixelsizeu', ...
    'Detector pixel size in direction U (lab unit/pixel)', 'double', 1};
list.detgeo(end+1, :) = {'pixelsizev', ...
    'Detector pixel size in direction V (lab unit/pixel)', 'double', 1};
list.detgeo(end+1, :) = {'detanglemin', ...
    '[Computed] Detector minimum 2Theta angle to consider', 'double', 0};
list.detgeo(end+1, :) = {'detanglemax', ...
    '[Computed] Detector maximum 2Theta angle to consider', 'double', 0};
list.detgeo(end+1, :) = {'detsizeu', ...
    'Number of pixels along the u direction', 'double', 1};
list.detgeo(end+1, :) = {'detsizev', ...
    'Number of pixels along the v direction', 'double', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sample geometry parameters for matching and indexing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.samgeo = cell(0, 4);
list.samgeo(end+1, :) = {'orig', ...
    'Lab coordinates of the origin of Sample reference', 'double', 0};
list.samgeo(end+1, :) = {'dirx', ...
    'Lab coordinates of Sample axis X (unit row vector)', 'double', 0};
list.samgeo(end+1, :) = {'diry', ...
    'Lab coordinates of Sample axis Y (unit row vector)', 'double', 0};
list.samgeo(end+1, :) = {'dirz', ...
    'Lab coordinates of Sample axis Z (unit row vector)', 'double', 0};
list.samgeo(end+1, :) = {'voxsize', ...
    'Voxel sizes of the Sample reference along X,Y,Z (lab unit/voxel); size(1,3)', 'double', 0};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reconstruction geometry parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.recgeo = cell(0, 4);
list.recgeo(end+1, :) = {'orig', ...
    'Lab coordinates of the origin of Reconstruction reference', 'double', 0};
list.recgeo(end+1, :) = {'dirx', ...
    'Lab coordinates of Reconstruction axis X (unit row vector)', 'double', 0};
list.recgeo(end+1, :) = {'diry', ...
    'Lab coordinates of Reconstruction axis Y (unit row vector)', 'double', 0};
list.recgeo(end+1, :) = {'dirz', ...
    'Lab coordinates of Reconstruction axis Z (unit row vector)', 'double', 0};
list.recgeo(end+1, :) = {'voxsize', ...
    'Voxel sizes of the Reconstruction reference along X,Y,Z (lab unit/voxel); size(1,3)', 'double', 0};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Preprocessing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.prep = cell(0, 4);
list.prep(end+1, :) = {'normalisation', ...
    'How to normalise images (''none''/''margin''/''fullbeam'')', 'char', 2};
list.prep(end+1, :) = {'absint', ...
    'Moving median interval in images for the direct beam (mod(total no. of projections, absint)=0)', 'double', 1};
list.prep(end+1, :) = {'absrange', ...
    'Moving median range in images for the direct beam (=n*2*absint)', 'double', 1};
list.prep(end+1, :) = {'fullint', ...
    'Moving median interval in images for the diffracted image (mod(total no. of projections, fullint)=0)', 'double', 1};
list.prep(end+1, :) = {'fullrange', ...
    'Moving median range in images for the diffracted image (=n*2*fullint)', 'double', 1};
list.prep(end+1, :) = {'margin', ...
    'Margin width for normalisation in direct beam (pixels)', 'double', 2};
list.prep(end+1, :) = {'intensity', ...
    'Assumed direct beam intensity of normalisation', 'double', 2};
list.prep(end+1, :) = {'filtsize', ...
    '2D median filter size for full images [pixels x pixels]', 'double', 2};
list.prep(end+1, :) = {'drifts_pad', ...
    'How to pad shifted images (''av'' or value)', 'char', 2};
list.prep(end+1, :) = {'renumbered', ...
    '[Computed] ...for interlaced scans', 'logical', 0};
list.prep(end+1, :) = {'bbox', ...
    '[Computed] Sample bounding box in the images', 'double', 0};
list.prep(end+1, :) = {'correct_drift', ...
    'we will shift images at the end of preprocessing, no shifting before flatfielding... (''required''/''not_required'')', 'char', 2};
list.prep(end+1, :) = {'udrift', ...
    '[Computed] Values for shifting images in U direction to compensate sample drifts', 'double', 0};
list.prep(end+1, :) = {'udriftabs', ...
    '[Computed] Values for shifting abs images in U direction to compensate sample drifts', 'double', 0};
list.prep(end+1, :) = {'vdrift', ...
    '[Computed] Values for shifting images in V direction to compensate sample drifts', 'double', 0};

% list.prep(end+1, :) = {'correct_drift_iteration', '', 'char' 2};
% list.prep(end+1, :) = {'prior_flatfield', 'Check if flatfielding is required', 'logical', 2};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Segmentation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.seg = cell(0, 4);
list.seg(end+1, :) = {'bbox', ...
    'Segmentation Bounding Box}; segmentation is done excluding this area', 'double', 1};
list.seg(end+1, :) = {'method', ...
    'Segmentation method: ''singlethr'' - single threshold}; ''doublethr'' - double threshold', 'char', 1};
list.seg(end+1, :) = {'thr_single', ...
    'Threshold for ''single threshold'' segmentation', 'double', 1};
list.seg(end+1, :) = {'thr_seed', ...
    'Seed threshold to find potential blobs (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'thr_grow_rat', ...
    'Relative intensity threshold for growing seeds (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'thr_grow_low', ...
    'Lower absolut limit of thr_grow_rat for growing seeds (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'thr_grow_high', ...
    'Upper absolut limit of thr_grow_rat for growing seeds (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'seedminarea', ...
    'Minimum seed size, in pixels, that will be considered (for double threshold). For the new segmentation it is hte number of connected pixels in 3D!', 'double', 1};
list.seg(end+1, :) = {'minsize', ...
    'Minimum accepted volume of a blob in voxels (for single threshold)', 'double', 1};
list.seg(end+1, :) = {'omintlimmin', ...
    'Min. relative intensity in omega image stack to create difspots from blobs', 'double', 1};
list.seg(end+1, :) = {'omintlimtail', ...
    'Relative integrated intensity for tail cut off to create difspots from blobs', 'double', 1};
list.seg(end+1, :) = {'minblobsize', ...
    'Minimum accepted blob size (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'maxblobsize', ...
    'Maximum accepted blob size (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'extendblobinc', ...
    'Size of incremental blob bbox extension (for double threshold)', 'double', 1};
list.seg(end+1, :) = {'background_subtract', ...
    'Offset the remaining median value of each full image to zero (computed and applied outside seg.bbox)', 'logical', 1};
list.seg(end+1, :) = {'overlaps_removed', ...
    'Removing of overlaps', 'logical', 1};
list.seg(end+1, :) = {'difspotmask', ...
    'Mask used to create difspot.edf from blob (''none''/''blob2D''/''blob3D'')', 'char', 1};
list.seg(end+1, :) = {'debug', ...
    'Display messages', 'logical', 1};
list.seg(end+1, :) = {'writeblobs', ...
    'Write difblobs to the table', 'logical', 1};
list.seg(end+1, :) = {'writespots', ...
    'Write difspot metadata to the table', 'logical', 1};
list.seg(end+1, :) = {'writeedfs', ...
    'Save difspots as edf files', 'logical', 1};
list.seg(end+1, :) = {'writehdf5', ...
    'Save difspots as hdf5 files', 'logical', 1};
list.seg(end+1, :) = {'wrapping', ...
    'For 360degree data, wrap from the last image back to the first image', 'logical', 1};
list.seg(end+1, :) = {'segmentation_stack_size', ...
    'How many images in memory? (1000 images approx 32Gb)', 'double', 1};
list.seg(end+1, :) = {'background_subtract_accelerate', ...
    'Calculate median on a subset of pixels (faster)', 'logical', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Friedel pair matching
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.match = cell(0, 4);
list.match(end+1, :) = {'thr_theta', ...
    'Max. theta angular deviation for a match (in degrees)', 'double', 1};
list.match(end+1, :) = {'thr_theta_scale', ...
    'Linear scaling for thr_theta: thr = thr_theta + thr_theta_scale*theta', 'double', 1};
list.match(end+1, :) = {'thr_max_offset', ...
    'Max. centroid image offset for a match (in no. of images)', 'double', 1};
list.match(end+1, :) = {'thr_ext_offset', ...
    'Max. offset of first and last image in omega stack (in no. of images)', 'double', 1};
list.match(end+1, :) = {'thr_genim_offset', ...
    'Max. offset for at least one of (centroid,first,last) image (in no. of images)', 'double', 1};
list.match(end+1, :) = {'thr_intint', ...
    'Max. intensity ratio for a match (>1)', 'double', 1};
list.match(end+1, :) = {'thr_area', ...
    'Max. area ratio for a match (>1)', 'double', 1};
list.match(end+1, :) = {'thr_bbsize', ...
    'Max. bbox size ratio (for both U and V) for a match (>1)', 'double', 1};
list.match(end+1, :) = {'minsizeu', ...
    'Min. bbox U size in pixels for spots to be considered for matching', 'double', 1};
list.match(end+1, :) = {'minsizev', ...
    'Min. bbox V size in pixels for spots to be considered for matching', 'double', 1};
list.match(end+1, :) = {'addconstr', ...
    'Additional constraints for spots to be considered (mysql command text)', 'char', 1};
list.match(end+1, :) = {'thr_meanerror', ...
    'Mean error under which a match is accepted', 'double', 1};
list.match(end+1, :) = {'thetalimits', ...
    'Allowed theta subranges in degrees: [min max] size (n,2)', 'double', 1};
list.match(end+1, :) = {'uniquetheta', ...
    'Theta tolerance for distinguishing overlapping {hkl} reflections (in degrees)', 'double', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Indexter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.index = cell(0, 4);
list.index(end+1, :) = {'strategy', ...
    'Strategy for indexter containing all parameters', 'struct', 0};
list.index(end+1, :) = {'discard', ...
    'Vector of pair id-s to be discarded in indexing', 'double', 1};
list.index(end+1, :) = {'forcemerge', ...
    'Cell array of grain id-s to be merged', 'cell', 1};

list.index__strategy = cell(0, 4);
list.index__strategy(end+1, :) = {'iter', ...
    'Number of iteration loops', 'double', 1};
list.index__strategy(end+1, :) = {'rfzext', ...
    'Extension of Rodrigues space for input reflections to account for measurement errors', 'double', 1};
list.index__strategy(end+1, :) = {'b', ...
    'Parameters for finding grains', 'struct', 0};
list.index__strategy(end+1, :) = {'m', ...
    'Parameters for merging grains', 'struct', 0};
list.index__strategy(end+1, :) = {'s', ...
    'Parameters for adding pairs to grains based on statistics', 'struct', 0};
list.index__strategy(end+1, :) = {'x', ...
    'Parameters for excluding pairs from grains based on statistics', 'struct', 0};

list.index__strategy__b = cell(0, 4);
list.index__strategy__b(end+1, :) = {'beg', ...
    'Tolerances in the first iteration loop for finding grains', 'struct', 0};
list.index__strategy__b(end+1, :) = {'end', ...
    'Tolerances in the last iteration loop for finding grains', 'struct', 0};

list.index__strategy__b__beg = cell(0, 4);
list.index__strategy__b__beg(end+1, :) = {'ang', ...
    'Max. angular deviation between two reflections in a grain (in degrees)', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'int', ...
    'Max. relative intensity ratio of two reflections in a grain', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'bbxs', ...
    'Max. relative bounding box X (U) size ratio in a grain', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'bbys', ...
    'Max. relative bounding box Y (V) size ratio in a grain', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'distf', ...
    'Max. distance between two diffraction paths (dmax = distf*grain_size)', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'distmax', ...
    'Max. absolut distance between two diffraction paths (in lab unit)', 'double', 1};
list.index__strategy__b__beg(end+1, :) = {'ming', ...
    'Min. no. of Friedel pairs in a grain', 'double', 1};

list.index__strategy__b__end = cell(0, 4);
list.index__strategy__b__end(end+1, :) = {'ang', ...
    'Max. angular deviation between two reflections in a grain (in degrees)', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'int', ...
    'Max. relative intensity ratio of two reflections in a grain', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'bbxs', ...
    'Max. relative bounding box X (U) size ratio in a grain', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'bbys', ...
    'Max. relative bounding box Y (V) size ratio in a grain', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'distf', ...
    'Max. relative distance between two diffraction paths (dmax = distf*grain_size)', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'distmax', ...
    'Max. absolut distance between two diffraction paths (in lab unit)', 'double', 1};
list.index__strategy__b__end(end+1, :) = {'ming', ...
    'Min. no. of Friedel pairs in a grain', 'double', 1};

list.index__strategy__m = cell(0, 4);
list.index__strategy__m(end+1, :) = {'beg', ...
    'Tolerances in the first iteration loop for grains to be merged', 'struct', 0};
list.index__strategy__m(end+1, :) = {'end', ...
    'Tolerances in the last iteration loop for grains to be merged', 'struct', 0};

list.index__strategy__m__beg = cell(0, 4);
list.index__strategy__m__beg(end+1, :) = {'bbxs', ...
    'Max. ratio of average bounding box X (U) size of grains', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'bbys', ...
    'Max. ratio of average bounding box Y (V) size of grains', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'int', ...
    'Max. ratio of average spot intensities of grains', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'distf', ...
    'Max. relative distance between grain centers (dmax = distf*grain_size)', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'distmin', ...
    'Min. absolut limit for distance between grain centers', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'distmax', ...
    'Max. absolut limit for distance between grain centers', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'angf', ...
    'Max. relative angular difference between grains in std (angmax = angf*ang_std_grain)', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'angmin', ...
    'Min. absolut limit for angular difference between grains', 'double', 1};
list.index__strategy__m__beg(end+1, :) = {'angmax', ...
    'Max. absolut limit for angular difference between grains', 'double', 1};

list.index__strategy__m__end = cell(0, 4);
list.index__strategy__m__end(end+1, :) = {'bbxs', ...
    'Max. ratio of average bounding box X (U) size of grains', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'bbys', ...
    'Max. ratio of average bounding box Y (V) size of grains', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'int', ...
    'Max. ratio of average spot intensities of grains', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'distf', ...
    'Max. relative distance between grain centers (dmax = distf*grain_size)', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'distmin', ...
    'Min. absolut limit for distance between grain centers', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'distmax', ...
    'Max. absolut limit for distance between grain centers', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'angf', ...
    'Max. relative angular difference between grains in std (angmax = angf*ang_std_grain)', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'angmin', ...
    'Min. absolut limit for angular difference between grains', 'double', 1};
list.index__strategy__m__end(end+1, :) = {'angmax', ...
    'Max. absolut limit for angular difference between grains', 'double', 1};

list.index__strategy__s = cell(0, 4);
list.index__strategy__s(end+1, :) = {'stdf', ...
    'Max. deviation from average grain properties for a new reflection to be included in grain (in std)', 'double', 1};

list.index__strategy__x = cell(0, 4);
list.index__strategy__x(end+1, :) = {'stdf', ...
    'Max. deviation from average grain properties for a reflection to be kept in grain (in std)', 'double', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% forward simulation of diffraction spots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.fsim = cell(0, 4);
list.fsim(end+1, :) = {'check_spot', ...
    'Check if there is not-segmented intensity in full images', 'logical', 1};
list.fsim(end+1, :) = {'omegarange', ...
    'Check for spots and intensity in +/- omega_range (deg)', 'double', 1};
list.fsim(end+1, :) = {'MaxOmegaOffset', ...
    'Maximum omega offset (deg)', 'double', 1};
list.fsim(end+1, :) = {'bb_size', ...
    'Dimensions of search bb - if left empty the projection size will be used', 'double', 1};
list.fsim(end+1, :) = {'assemble_figure', ...
    'Assemble the results of forward simulation into a full image saved in the grain_04%d.mat file', 'logical', 1};
list.fsim(end+1, :) = {'display_figure', ...
    'Display the assembled full image (function gtShowFsim)', 'logical', 1};
list.fsim(end+1, :) = {'clims', ...
    'Colorlimits for display of results', 'double', 1};
list.fsim(end+1, :) = {'Rdist_factor', ...
    'Allow for Rdist_factor times dangstd in calulcation of angular deviation', 'double', 1};
list.fsim(end+1, :) = {'bbsize_factor', ...
    'Allow for bbsize_factor times bbx(y)sstd variation of BoundingBox size', 'double', 1};
list.fsim(end+1, :) = {'oversize', ...
    'Projections are zeropadded to this size before reconstruction', 'double', 1};
list.fsim(end+1, :) = {'use_th', ...
    'Use theoretically predicted diffraction angles for new detected spots', 'logical', 1};
list.fsim(end+1, :) = {'verbose', ...
    'Display the search criteria for each of the spots', 'logical', 1};
list.fsim(end+1, :) = {'save_grain', ...
    'Flag to save grain_####.mat files', 'logical', 1};
list.fsim(end+1, :) = {'oversizeVol', ...
    'Grain reconstruction volumes are zeropadded to this size', 'double', 1};

list.fsim(end+1, :) = {'sel_lh_stdev', ...
    'Selects spots for reconsruction that are in the top 4.5 sigma of the likelihood rankings', 'double', 1};
list.fsim(end+1, :) = {'sel_int_stdev', ...
    'Selects spots for reconsruction that are in the top 4 sigma of the intensity rankings', 'double', 1};
list.fsim(end+1, :) = {'sel_avint_stdev', ...
    'Selects spots for reconsruction that are in the top 3.5 sigma of the average intensity per pixel rankings', 'double', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reconstruction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.rec = cell(0, 4);
list.rec(end+1, :) = {'absorption', ...
    'Absorption volume reconstruction information', 'struct', 0};
list.rec(end+1, :) = {'grains', ...
    'Grains volume reconstruction information', 'struct', 0};
list.rec(end+1, :) = {'thresholding', ...
    'Grains volume segmentation information', 'struct', 0};

list.rec__absorption = cell(0, 4);
list.rec__absorption(end+1, :) = {'algorithm', ...
    'Algorithm for the absorption reconstruction {''SIRT''}/''2DFBP''/''3DTV''', 'char', 1};
list.rec__absorption(end+1, :) = {'num_iter', ...
    'Number of iterations used in SIRT reconstruction of absorption volume', 'double', 1};
list.rec__absorption(end+1, :) = {'interval', ...
    'Interval between radiographs used for reconstruction of absorption scan', 'double', 1};
list.rec__absorption(end+1, :) = {'padding', ...
    'Pixels padding around the projection data and volume', 'double', 1};
list.rec__absorption(end+1, :) = {'options', ...
    'Reconstruction algorithm''s options', 'struct', 0};

list.rec__grains = cell(0, 4);
list.rec__grains(end+1, :) = {'algorithm', ...
    'Algorithm for the absorption reconstruction {''SIRT''}/''3DTV''/''6DL1''/''6DTV''/''6DTVL1''', 'char', 1};
list.rec__grains(end+1, :) = {'num_iter', ...
    'Number of iterations used in the reconstruction of each grain volume', 'double', 1};
list.rec__grains(end+1, :) = {'list', ...
    'list of grain IDs to be reconstructed', 'double', 1};
list.rec__grains(end+1, :) = {'options', ...
    'Reconstruction algorithm''s options', 'struct', 0};

list.rec__thresholding = cell(0, 4);
list.rec__thresholding(end+1, :) = {'percentile', ...
    'Threshold: percentile/100 * std', 'double', 1};
list.rec__thresholding(end+1, :) = {'do_morph_recon', ...
    'Perform morphological reconstruction during segmentation', 'logical', 1};
list.rec__thresholding(end+1, :) = {'num_iter', ...
    'Number of iterations used in thresholding grains'' volume', 'double', 1};
list.rec__thresholding(end+1, :) = {'iter_factor', ...
    'Weight factor for the iterative segmentation', 'double', 1};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.fed = cell(0, 4);
list.fed(end+1, :) = {'dir', ...
    'Data analysis directory', 'char', 1};
list.fed(end+1, :) = {'dct_vol', ...
    'Dct reconstructed volume filename (.edf)', 'char', 1};
list.fed(end+1, :) = {'dct_offset', ...
    'Offset to be applied to the volume [x y z]', 'double', 1};
list.fed(end+1, :) = {'renumber_list', ...
    'Renumbering list for grainID', 'double', 1};



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% oar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
list.oar = cell(0, 4);
list.oar(end+1, :) = {'njobs', ...
    'Number of OAR jobs?', 'double', 1};
list.oar(end+1, :) = {'walltime', ...
    'Walltime for OAR? (in seconds)', 'double', 1};
list.oar(end+1, :) = {'mem', ...
    'Total memory required? (Mb)', 'double', 1};

end % end of function

