'''
Created on Jun 11, 2013

@author: ben
'''

import smtplib
import sys

from . import dct_utils_platform

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

class DCTNotify(object):

    def notify_myself(self, subject, message = ""):
        raise NotImplementedError("Use one of the derived classes")


class DCTNotifyEmail(DCTNotify):

    def __init__(self, conf, address = ""):
        self.conf = conf
        if address == "":
            address = self._get_address()
        self.address = address

    def _get_smtp(self):
        smtp_server = self.conf.getSmtpServer()
        print("Using server: %s" % smtp_server)
        return smtplib.SMTP(smtp_server)

    def _get_address(self):
        addr = self.conf.getUserEmail()
        if addr == "":
            #: Let's guess if in the ESRF
            utils = dct_utils_platform.UtilsFactory.getMachineDep()
            addr = utils.guessUserEmail()
        return addr

    def _create_message(self, from_user, to_user, subject, message, attachments):
        if len(attachments) > 0:
            msg = MIMEMultipart()
            msg.attach(MIMEText(message, 'plain'))
        else:
            msg = MIMEText(message)
        msg['Subject'] = subject
        msg['From'] = from_user
        msg['To'] = to_user
        for a in attachments:
            msg.attach(a)
        return msg

    def _send_msg(self, server, msg):
        if sys.version_info[0] == 2:
            server.sendmail(msg["From"], [msg["To"]], msg.as_string())
        else:
            server.send_message(msg)
        server.quit()

    def make_attachment_textfile(self, filename):
        fid = open(filename);
        attach = MIMEText(fid.read())
        attach.add_header('Content-Disposition', 'attachment', filename=filename)
        fid.close()
        return attach

    def notify_myself(self, subject, message = "", attachments = []):
        server = self._get_smtp()
        msg = self._create_message(from_user=self.address, \
                                   to_user=self.address, \
                                   subject=subject, message=message, \
                                   attachments=attachments)
        self._send_msg(server, msg)

    def notify_graintracking(self, subject, message, attachments = []):
        server = self._get_smtp()
        msg = self._create_message(from_user=self.address, \
                                   to_user="graintracking@esrf.fr", \
                                   subject=subject, message=message, \
                                   attachments=attachments)
        self._send_msg(server, msg)


