classdef GtOrientationVolView < GtVolView
    properties
        r_vecs = [];
    end

    methods (Access = public)
        function self = GtOrientationVolView(r_vecs, intensities, varargin)
            self = self@GtVolView(intensities, varargin{:});

            % bringing average to 0, so to avoid things that were 0 in the
            % first place
%             mask = intensities > (max(intensities(:)) / 100);
            mask = intensities > 0;
            size_ints = [size(mask, 1), size(mask, 2), size(mask, 3)];
            zeros_ints = false(size_ints);
            mask1 = cat(4, mask, zeros_ints, zeros_ints);
            mask2 = cat(4, zeros_ints, mask, zeros_ints);
            mask3 = cat(4, zeros_ints, zeros_ints, mask);
            avg = [mean(r_vecs(mask1)), mean(r_vecs(mask2)), mean(r_vecs(mask3))];

            r_vecs(mask1) = r_vecs(mask1) - avg(1);
            r_vecs(mask2) = r_vecs(mask2) - avg(2);
            r_vecs(mask3) = r_vecs(mask3) - avg(3);

            r_vecs(:, :, :, 1) = rescale(r_vecs(:, :, :, 1));
            r_vecs(:, :, :, 2) = rescale(r_vecs(:, :, :, 2));
            r_vecs(:, :, :, 3) = rescale(r_vecs(:, :, :, 3));

            self.r_vecs = r_vecs;

            function x = rescale(x)
                x = (x - min(x(:))) / (max(x(:)) - min(x(:)));
            end
        end
    end

    methods (Access = protected)
        function initGui(self)
            self.initGui@GtVolView();

            self.conf.h_im = mesh('Parent', self.conf.h_ax, [0 0; 0 0], ...
                'EdgeColor', [0.2 0.2 0.2], 'FaceColor', 'interp', ...
                'EdgeAlpha', 0.5);
            self.conf.h_overlay = [];
        end

        function slice = loadSlice(self, vol)
        % GTVOLVIEW/LOADSLICE Loads a slice from the volume (given the plane and
        % slice index in obj.conf.*) and permutes it to be visualised correctly
        % by matlab, which always flips X and Y dimensions in visualisation.

            slice(:, :, 4) = self.loadSlice@GtVolView(vol);
            slice(:, :, 3:4) = smooth3(slice(:, :, 3:4), 'box', [5 5 1]);

            if (~isempty(self.r_vecs))
                slice(:, :, 1) = self.loadSlice@GtVolView(self.r_vecs(:, :, :, 1));
                slice(:, :, 2) = self.loadSlice@GtVolView(self.r_vecs(:, :, :, 2));
                slice(:, :, 3) = self.loadSlice@GtVolView(self.r_vecs(:, :, :, 3));
            end
        end

        function setVisualisedData(self, image, vol, is_overlay)
        % GTVOLVIEW/SETVISUALISEDDATA Loads a slice and then operates
        % transformations on them, to adapt to the requested visualization needs

            slice = self.loadSlice(vol);
            slice = double(slice);

            if (is_overlay && ~isa(vol,'integer'))
                sliceMin = min(slice(:));
                sliceMax = max(slice(:));
                sliceRange = sliceMax - sliceMin;
                slice = repmat(slice-sliceMin, [1,1,3])/sliceRange;
            end

            avg_R_vecs = slice(:, :, 1:3);
            avg_ints = slice(:, :, 4);

            % Rescaling
            avg_ints = avg_ints / max(self.conf.vol(:)) ...
                * (size(avg_ints, 1) + size(avg_ints, 2)) / 6;

%             if (~isempty(self.r_vecs))
%                 avg_R_vecs(:, :, 1) = rescale(avg_R_vecs(:, :, 1), self.r_vecs(:, :, :, 1));
%                 avg_R_vecs(:, :, 2) = rescale(avg_R_vecs(:, :, 2), self.r_vecs(:, :, :, 2));
%                 avg_R_vecs(:, :, 3) = rescale(avg_R_vecs(:, :, 3), self.r_vecs(:, :, :, 3));
%             end

            set(image, 'CData', avg_R_vecs);
            set(image, 'ZData', avg_ints);
        end
    end
end